/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Bluetooth.java
 *
 * Created on November 12, 2004, 1:59 PM
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Bluetooth.java,v 1.7 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.bluetooth;

import java.io.OutputStream;
import java.util.*;

import javax.bluetooth.*;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

import birg.bluemove.communication.core.*;
import org.apache.log4j.Logger;


/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class handles the Bluetooth communication and devices
 * discovery. It does not need a complete services implementation. Instead, we
 * use this magic url: btspp://424952470001:1. It allows to directly create a
 * SPP communication.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class Bluetooth extends DataInterface {

    static Logger logger = Logger.getLogger(Bluetooth.class);

    static private Bluetooth m_instance = null;
    private LocalDevice m_localDev = null;
    private HashMap<String, OutputStream> m_osMap = null;
    private DiscoveryAdapter m_adapter = null;

    /**
     * This constructor is private because Bluetooth is a singleton. It throws
     * an exception when no local device is found.
     *
     * @throws BluetoothStateException
     */
    private Bluetooth() throws BluetoothStateException {
        logger.debug("Creating a new Bluetooth instance...");
        m_osMap = new HashMap<String, OutputStream>();
        m_adapter = new DiscoveryAdapter();
    }

    /**
     * Allows to get the Bluetooth reference. This method must be used instead
     * of the standard constructor.
     *
     * @return Bluetooth
     * @throws BluetoothStateException
     */
    static public Bluetooth getInstance() throws BluetoothStateException {
        if (m_instance == null) {
            m_instance = new Bluetooth();
        }
        return m_instance;
    }


    /**
     * Uses the ModuleDataManager to send data to all the modules present in the
     * manager. Currently, it only handles one actuator per module. A "standard"
     * format should be defined.
     *
     * @param manager ModuleDataManager
     * @return boolean
     */
    public boolean sendData(ModuleDataManager manager) {
        logger.debug("Sending data...");
        for (ModuleData element : manager.getModuleList()) {
            String address = element.getAddress();
            // The modules have only one servo (need one position).
            ActuatorData servo = (ActuatorData) element.getComponent("Servo1");
            // Check is this actuator is available in the module
            if (servo == null) {
                // No actuator define with this name
                // so continue with another
                continue;
            }

            int position = (int) servo.getValue();

            logger.debug("BTAddress -> " + address);
            logger.debug("Position  -> " + position);

            transmitData(address, position);
        }
        return true;
    }

    /**
     * Starts the discovery of Bluetooth device which are in the local device
     * range.
     */
    public void inquiry() {
        logger.debug("Start inquiring...");
        m_moduleList.clear();
        try {
            m_adapter.startInquiry(DiscoveryAgent.GIAC);
            LinkedList<RemoteDevice> devs = m_adapter.waitDevices();
            for (RemoteDevice element : devs) {
                logger.debug("Adding " + element.getBluetoothAddress());
                m_moduleList.add(new ModuleData(element.getBluetoothAddress()));
            }
        } catch (BluetoothStateException e) {
            logger.error(e.getMessage());
        }
    }

    /**
     * Stops the inquiry if needed.
     */
    public void stopInquiry() {
        logger.debug("Stop inquiring...");
        m_adapter.stopInquiry();
    }

    /**
     * Filters the modules addresses in order to only keep the ones which matche
     * the pattern.
     *
     * @param addresses LinkedList
     * @param pattern String
     * @return LinkedList
     */
    public LinkedList<String> filterModulesList(LinkedList<String> addresses,
            String pattern) {
        LinkedList<String> addressList = new LinkedList<String>();
        for (String element : addresses) {
            if (element.substring(0, 8).equals(pattern)) {
                addressList.add(element);
            }
        }
        return addressList;
    }

    /**
     * Constructs the "magic" URL which allows to create a SPP connection with a
     * module.
     *
     * @param address String
     * @return String
     */
    private String constructURL(String address) {
        return "btspp://" + address + ":1";
    }

    /**
     * Gets an output stream for a given module. This output stream allows then
     * to send data to the module.
     *
     * @param address String
     * @return OutputStream
     */
    public OutputStream getOutputStream(String address) {
        OutputStream outStream = null;
        String url = constructURL(address);
        try {
            logger.info("Creating connection for " + address);
            StreamConnection conn = (StreamConnection) Connector.open(url);
            outStream = conn.openOutputStream();
            conn.close();
        } catch (Exception e) {
            logger.error(e.getMessage());
            logger.error("Please check that your Bluetooth device is ready.");
            return null;
//            System.exit(1);
        }
        return outStream;
    }

    /**
     * Starts a SPP connection for all the modules present in the manager.
     *
     * @param manager ModuleDataManager
     * @return boolean
     */
    public boolean startConnections(ModuleDataManager manager) {
        logger.debug("Start connections...");
        for (ModuleData element : manager.getModuleList()) {
            String address = element.getAddress();
            m_osMap.put(address, getOutputStream(address));
        }
        return true;
    }

    /**
     * Stops all the connections which were opened by startConnections().
     *
     * @return boolean
     */
    public boolean stopConnections() {
        logger.debug("Stop connections...");
        Collection<OutputStream> osMapColl = m_osMap.values();
        for (OutputStream os : osMapColl) {
            try {
                os.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return true;
    }

    /**
     * Transmits the data to the module. It checks that |position| < 100. This
     * is AMR dependant and should be removed.
     *
     * @param address String
     * @param position int
     * @return boolean
     */
    private boolean transmitData(String address, int position) {
        logger.debug("Transmiting data...");

        byte[] data = new byte[2];

        OutputStream os = m_osMap.get(address);

        data[0] = -1;
        if (position > 100) {
            data[1] = 100;
        } else if (position < -100) {
            data[1] = -100;
        } else {
            data[1] = (byte) position;
        }

        logger.debug("Data: " + data[0] + "," + data[1]);
        try {
            os.write(data);
            os.flush();
        } catch (Exception e) {
            logger.debug(e.getMessage());
        }
        return true;
    }
}
