/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * DiscoveryAdapter.java
 *
 * Created on November 16, 2004, 11:27 AM
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: DiscoveryAdapter.java,v 1.4 2005/01/25 23:57:20 jaquier Exp $
 */

package birg.bluemove.communication.bluetooth;

import java.util.LinkedList;

import javax.bluetooth.*;

import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is an "helper" class. It handles basic Bluetooth
 * operations like inquiring. A DiscoveryAdapter is already present in the
 * Rococo API but it is not JSR-82 compliant. So we just implement our own
 * DiscoveryAdapter.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class DiscoveryAdapter implements DiscoveryListener {

    static Logger logger = Logger.getLogger(DiscoveryAdapter.class);

    private LocalDevice localDev = null;
    private DiscoveryAgent agent = null;
    private LinkedList<RemoteDevice> devs = null;

    private Boolean isInquiring;

    /**
     * Constructor. Gets the local device reference and an agent from this
     * device. Creates the initial list which contains the discovered devices.
     *
     * @throws BluetoothStateException
     */
    public DiscoveryAdapter() throws BluetoothStateException {
        // Sets the mutex to false.
        isInquiring = false;
        // Gets a reference of the local device.
        localDev = LocalDevice.getLocalDevice();
        // Gets an agent.
        agent = localDev.getDiscoveryAgent();
        // Initializes the device list.
        devs = new LinkedList<RemoteDevice>();
    }

    /**
     * Starts to inquiry for available devices.
     *
     * @param accessCode int
     * @throws BluetoothStateException
     */
    public void startInquiry(int accessCode) throws BluetoothStateException {
        synchronized (isInquiring) {
            devs.clear();
            isInquiring = true;
            agent.startInquiry(accessCode, this);
        }
    }

    /**
     * Gets the inquiry state.
     *
     * @return boolean
     */
    public boolean isInquiring() {
        synchronized (isInquiring) {
            return isInquiring;
        }
    }

    /**
     * This method is called by the listener when a device is discovered. We
     * just add it to the list.
     *
     * @param btDevice RemoteDevice
     * @param cod DeviceClass
     */
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod) {
        logger.debug("Found a device: " + btDevice.getBluetoothAddress());
        // Adds RemoteDevice to the device list.
        devs.add(btDevice);
    }

    /**
     * This method is called when the inquiry is finished.
     *
     * @param discType int
     */
    public void inquiryCompleted(int discType) {
        synchronized (isInquiring) {
            isInquiring = false;
            if (discType == DiscoveryListener.INQUIRY_TERMINATED) {
                logger.debug("Application terminated search");
            } else if (discType == DiscoveryListener.INQUIRY_COMPLETED) {
                logger.debug("Device inquiry terminated normally");
            } else if (discType == DiscoveryListener.INQUIRY_ERROR) {
                logger.debug("Error performing device inquiry");
            }
        }
    }

    /**
     * Stops the inquiry. <pre>InquiryCompleted</pre> will be called.
     */
    public void stopInquiry() {
        logger.debug("Stopping inquiring...");
        agent.cancelInquiry(this);
    }

    /**
     * Waits while the Bluetooth device is inquiring. Then return the discovered
     * devices.
     *
     * @return LinkedList
     */
    public LinkedList<RemoteDevice> waitDevices() {
        while (isInquiring()) {
            try {
                // We sleep instead of just looping.
                Thread.sleep(500);
            } catch (InterruptedException e) {
                logger.error(e.getMessage());
            }
        }
        return devs;
    }

    /**
     * We do not care about services.
     *
     * @param transID int
     * @param servRecord ServiceRecord[]
     */
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {
        // Not needed
    }

    /**
     * We do not care about services.
     *
     * @param transID int
     * @param respCode int
     */
    public void serviceSearchCompleted(int transID, int respCode) {
        // Not needed
    }
}
