/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * XMLConfig.java
 *
 * Created on 23. novembre 2004, 12:46
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: XMLConfig.java,v 1.5 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.io.*;
import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerException;

import com.sun.org.apache.xpath.internal.XPathAPI;
import org.apache.log4j.Logger;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.*;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class defines basic methods for XML handling. We use a
 * DOM parser and a XPath engine from Apache.org. This class should not be used
 * directly but should be extended.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class XMLConfig {

    static Logger logger = Logger.getLogger(XMLConfig.class);

    protected Document m_document = null;
    protected Config m_config = null;
    protected String m_fName = null;
    private DocumentBuilder m_builder = null;

    /**
     * Constructor. Gets a Document reference which allows to parse XML document
     * and to create new nodes.
     *
     * @param config Config
     * @param fName String
     * @param load boolean
     */
    public XMLConfig(Config config, String fName, boolean load) {
        try {
            m_config = config;
            m_fName = fName;
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            m_builder = factory.newDocumentBuilder();
            m_document = m_builder.newDocument();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Reads the XML file from the archive and creates a DOM tree.
     *
     * @return boolean
     */
    public boolean parseFile() {
        InputStream iStream = m_config.readFromArchive(m_fName);
        if (iStream != null) {
            try {
                m_document = m_builder.parse(iStream);
            } catch (Exception e) {
                logger.error(e.getMessage());
                return false;
            }
            m_document.getDocumentElement().normalize();
            return true;
        }
        return false;
    }

    /**
     * Returns a list of items matching the XPath pattern.
     *
     * @param xpath String
     * @return LinkedList
     * @todo Remove the empty string value in case of no matching and return a
     *   null value which must be handled correctly upstream.
     */
    public LinkedList<String> getValueFromXPath(String xpath) {
        LinkedList<String> sList = new LinkedList<String>();
        try {
            NodeList nList = XPathAPI.selectNodeList(m_document, xpath);
            for (int i = 0; i < nList.getLength(); i++) {
                if (nList.item(i).hasChildNodes()) {
                    sList.add(nList.item(i).getFirstChild().getNodeValue());
                } else {
                    // Add an empty string value
                    sList.add("");
                }
            }
        } catch (TransformerException e) {
            logger.error(e.getMessage());
        }
        return sList;
    }

    /**
     * Returns a node from the XPath pattern.
     *
     * @param xpath String
     * @return Node
     */
    private Node getNodeFromXPath(String xpath) {
        Node node = null;
        try {
            node = XPathAPI.selectSingleNode(m_document, xpath);
        } catch (TransformerException e) {
            logger.error(e.getMessage());
        }
        return node;
    }

    /**
     * Sets a node value using a XPath pattern to select the node.
     *
     * @param xpath String
     * @param value String
     */
    public void setNodeValue(String xpath, String value) {
        Node node = getNodeFromXPath(xpath);
        node.setNodeValue(value);
    }

    /**
     * Sets a text value using a XPath pattern to select the node.
     *
     * @param xpath String
     * @param value String
     */
    public void setTextValue(String xpath, String value) {
        Node node = getNodeFromXPath(xpath);
        node.appendChild(m_document.createTextNode(value));
    }

    /**
     * Adds a new node to a node selected with XPath pattern.
     *
     * @param xpath String
     * @param nName String
     */
    public void addNode(String xpath, String nName) {
        Node node = getNodeFromXPath(xpath);
        Element current = m_document.createElement(nName);
        node.appendChild(current);
    }

    /**
     * Adds a parameter (name, value) to an existing node which matches the
     * XPath pattern.
     *
     * @param xpath String
     * @param pName String
     * @param pValue String
     */
    public void addNodeParameter(String xpath, String pName, String pValue) {
        Element node = (Element) getNodeFromXPath(xpath);
        node.setAttribute(pName, pValue);
    }

    /**
     * Saves the XML DOM tree into the archive.
     *
     * @todo Maybe this method should be not here or modified.
     */
    public void saveToFile() {
        OutputStream oS = new ByteArrayOutputStream();
        OutputFormat oF = new OutputFormat("XML", "ISO-8859-1", true);
        oF.setIndent(1);
        oF.setIndenting(true);

        XMLSerializer xmlS = new XMLSerializer(oS, oF);
        try {
            xmlS.asDOMSerializer();
            xmlS.serialize(m_document);
            m_config.writeToArchive(oS, m_fName);
            oS.close();
        } catch (IOException e) {
            logger.error(e.getMessage());
        }
    }

    /**
     * Gets the Document instance. Needed for node creation.
     *
     * @return Document
     */
    public Document getDocument() {
        return m_document;
    }
}
