/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * XMLModuleManager.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: XMLModuleManager.java,v 1.6 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.util.LinkedList;

import birg.bluemove.module.core.*;
import org.w3c.dom.*;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is used to save and restore the modules manager.
 * It converts the modules manager singleton into a XML file which will be store
 * into the project archive. It can also recreates the modules manager from the
 * XML file.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class XMLModuleManager extends XMLConfig implements Persistance {

    ModulesManager m_mManager = ModulesManager.getInstance();

    /**
     * Constructor. We use the file named <pre>config/modules.xml</pre> to store
     * and restore the data.
     *
     * @param config Config
     */
    public XMLModuleManager(Config config) {
        super(config, "config/modules.xml", true);
    }

    /**
     * Reads the data from the XML file.
     */
    public void get() {
        m_mManager.clear();
        if (parseFile()) {
            getModules();
            getDescription();
        }
    }

    /**
     * Gets the project description.
     */
    private void getDescription() {
        LinkedList<String> desc = null;
        desc = getValueFromXPath("/bluemove/modules/description");
        m_mManager.setDescription(desc.getFirst());
    }

    /**
     * Restores all the modules present in the XML file.
     */
    private void getModules() {
        LinkedList<String> mList = null;
        LinkedList<String> temp = null;

        mList = getValueFromXPath("/bluemove/modules/*/@name");
        for (String element : mList) {
            try {
                Module m = new Module(element);

                // Address
                temp = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                         element + "\"]/address/@value");
                m.setAddress(temp.getFirst());

                // Comment
                temp = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                         element + "\"]/comment");
                m.setComment(temp.getFirst());

                m_mManager.addModule(m);

                getComponents(m);
            } catch (Exception e) {
                logger.error(e.getMessage());
            }
        }
    }

    /**
     * Reloads the components which belongs to the module <pre>module</pre>.
     *
     * @param module Module
     */
    private void getComponents(Module module) {
        LinkedList<String> comp = null;

        comp = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                 module.getName() +
                                 "\"]/components/actuator/@name");
        for (String element : comp) {
            try {
                Actuator actu = new Actuator(module, element);
                actu.setRange(getActuatorRange(module.getName(), element));
                module.addComponent(actu);
            } catch (Exception e) {
                logger.error(e.getMessage());
            }
        }

        comp = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                 module.getName() +
                                 "\"]/components/sensor/@name");
        for (String element : comp) {
            try {
                Sensor sens = new Sensor(module, element);
                module.addComponent(sens);
            } catch (Exception e) {
                logger.error(e.getMessage());
            }
        }
    }

    /**
     * Gets the range of the given actuator.
     *
     * @param mName String
     * @param aName String
     * @return Double[]
     */
    private double[] getActuatorRange(String mName, String aName) {
        String xpath = null;

        xpath = "/bluemove/modules/module[@name=\"" + mName +
                "\"]/components/actuator[@name=\"" + aName + "\"]/@min";
        Double min = Double.parseDouble(getValueFromXPath(xpath).getFirst());

        xpath = "/bluemove/modules/module[@name=\"" + mName +
                "\"]/components/actuator[@name=\"" + aName + "\"]/@max";
        Double max = Double.parseDouble(getValueFromXPath(xpath).getFirst());

        return new double[] {min, max};
    }

    /**
     * Saves the modules manager to the XML file.
     */
    public void set() {
        Element tempNodeA = null;
        Element tempNodeB = null;

        tempNodeA = m_document.createElement("bluemove");
        m_document.appendChild(tempNodeA);
        tempNodeB = m_document.createElement("modules");
        setDescription(tempNodeB);
        tempNodeA.appendChild(tempNodeB);
        setModules(tempNodeB);

        saveToFile();
    }

    /**
     * Saves the project description.
     *
     * @param root Node
     */
    private void setDescription(Node root) {
        Element temp = m_document.createElement("description");
        Text text = m_document.createCDATASection(m_mManager.getDescription());
        temp.appendChild(text);
        root.appendChild(temp);
    }

    /**
     * Saves the modules from the modules manager.
     *
     * @param root Node
     */
    private void setModules(Node root) {
        LinkedList<Module> mList = m_mManager.getModuleList();
        for (Module element : mList) {
            Element tempNodeA = m_document.createElement("module");
            tempNodeA.setAttribute("name", element.getName());
            root.appendChild(tempNodeA);

            Element tempNodeB = m_document.createElement("address");
            tempNodeB.setAttribute("value", element.getAddress());
            tempNodeA.appendChild(tempNodeB);

            tempNodeB = m_document.createElement("comment");
            Text text = m_document.createCDATASection(element.getComment());
            tempNodeB.appendChild(text);
            tempNodeA.appendChild(tempNodeB);

            tempNodeB = m_document.createElement("components");
            tempNodeA.appendChild(tempNodeB);
            setComponents(element, tempNodeB);
        }
    }

    /**
     * Saves the components from the given module.
     *
     * @param current Module
     * @param root Node
     */
    private void setComponents(Module current, Node root) {
        for (ModuleComponent element : current.getComponentList()) {
            Element comp = null;
            if (element instanceof Actuator) {
                comp = m_document.createElement("actuator");

                Double min = ((Actuator) element).getRangeMin();
                Double max = ((Actuator) element).getRangeMax();
                comp.setAttribute("min", min.toString());
                comp.setAttribute("max", max.toString());
            } else if (element instanceof Sensor) {
                comp = m_document.createElement("sensor");
            }
            comp.setAttribute("name", element.getName());
            root.appendChild(comp);
        }
    }
}
