/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * XMLModuleManager.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: XMLTimelinesManager.java,v 1.2 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.util.LinkedList;

import birg.bluemove.module.core.*;
import birg.bluemove.timelines.core.curve.ActuatorCurve;
import birg.bluemove.timelines.core.interpolator.Key;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is used to save and restore the timelines.
 * It uses the modules manager singleton to retrieve timeline objects and
 * serializes them into a XML file which will be store into the project archive.
 * It can also recreates the timelines from the XML file. The main data saved
 * here are the timelines keys. The timelines loading <strong>must</strong> be
 * done after the modules manager loading (XMLModuleManager).</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class XMLTimelinesManager extends XMLConfig implements Persistance {

    ModulesManager m_mManager = ModulesManager.getInstance();

    /**
     * Constructor. The timelines data are stored in a file called
     * <pre>config/timelines.xml</pre>. This file is stored in the archive.
     *
     * @param config Config
     */
    public XMLTimelinesManager(Config config) {
        super(config, "config/timelines.xml", true);
    }

    /**
     * Restores the timelines data. We need to restore the modules manager
     * <strong>before</strong> calling this method
     */
    public void get() {
        if (parseFile()) {
            getModules();
        }
    }

    /**
     * Gets the timeline data from all modules in the modules manager.
     */
    private void getModules() {
        LinkedList<String> mList = null;
        LinkedList<String> temp = null;

        mList = getValueFromXPath("/bluemove/modules/*/@name");
        for (String element : mList) {
            Module m = m_mManager.getModule(element);

            // isActive
            temp = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                     element + "\"]/@active");
            m.getTimelineModule().setActive(Boolean.parseBoolean(temp.getFirst()));

            getComponents(m);
        }
    }

    /**
     * Gets the timeline data from the components. Here we get all the keys
     * defined in a timeline.
     *
     * @param module Module
     */
    private void getComponents(Module module) {
        LinkedList<String> comp = null;
        LinkedList<String> type = null;
        LinkedList<String> frame = null;
        LinkedList<String> value = null;
        boolean isActive = false;

        String moduleName = module.getName();

        comp = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                 moduleName +
                                 "\"]/components/actuator/@name");

        for (String element : comp) {
            isActive = Boolean.parseBoolean(getValueFromXPath(
                    "/bluemove/modules/module[@name=\"" + moduleName +
                    "\"]/components/actuator[@name=\"" + element +
                    "\"]/@active").getFirst());

            type = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                     moduleName +
                                     "\"]/components/actuator[@name=\"" +
                                     element + "\"]/curve/key/@type");

            frame = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                      moduleName +
                                      "\"]/components/actuator[@name=\"" +
                                      element + "\"]/curve/key/@frame");

            value = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                      moduleName +
                                      "\"]/components/actuator[@name=\"" +
                                      element + "\"]/curve/key/@value");

            Actuator actu = (Actuator) module.getComponent(element);
            ActuatorCurve acurve = (ActuatorCurve) actu.getCurve();

            acurve.setActive(isActive);
            for (int i = 0; i < type.size(); i++) {
                double frameValue = Double.parseDouble(frame.get(i));
                double valueValue = Double.parseDouble(value.get(i));
                Key.KeyType keyType = Key.KeyType.valueOf(type.get(i));

                Key key = new Key(keyType, frameValue, valueValue);

                acurve.addKey(key);
            }
        }

        comp = getValueFromXPath("/bluemove/modules/module[@name=\"" +
                                 module.getName() +
                                 "\"]/components/sensor/@name");
        for (String element : comp) {
//            Sensor sens = new Sensor(module, element);
//            module.addComponent(sens);
        }
    }

    /**
     * Saves the timelines data into the XML file.
     */
    public void set() {
        Element tempNodeA = null;
        Element tempNodeB = null;

        tempNodeA = m_document.createElement("bluemove");
        m_document.appendChild(tempNodeA);
        tempNodeB = m_document.createElement("modules");
        tempNodeA.appendChild(tempNodeB);
        setModules(tempNodeB);

        saveToFile();
    }

    /**
     * Sets the timelines data in the module point of view.
     *
     * @param root Node
     */
    private void setModules(Node root) {
        LinkedList<Module> mList = m_mManager.getModuleList();
        for (Module element : mList) {
            Element tempNodeA = m_document.createElement("module");
            tempNodeA.setAttribute("name", element.getName());
            boolean isActive = element.getTimelineModule().isActive();
            tempNodeA.setAttribute("active", Boolean.toString(isActive));
            root.appendChild(tempNodeA);

            Element tempNodeB = m_document.createElement("components");
            tempNodeA.appendChild(tempNodeB);
            setComponents(element, tempNodeB);
        }
    }

    /**
     * Sets the timelines data in the components point of view.
     *
     * @param current Module
     * @param root Node
     */
    private void setComponents(Module current, Node root) {
        for (ModuleComponent element : current.getComponentList()) {
            Element comp = null;
            if (element instanceof Actuator) {
                ActuatorCurve acurve = (ActuatorCurve) element.getCurve();
                comp = m_document.createElement("actuator");
                comp.setAttribute("active", Boolean.toString(acurve.isActive()));
                setCurve((Actuator) element, comp);
            } else if (element instanceof Sensor) {
                comp = m_document.createElement("sensor");
            }
            comp.setAttribute("name", element.getName());
            root.appendChild(comp);
        }
    }

    /**
     * Sets the curve for an actuator component.
     *
     * @param current Actuator
     * @param root Node
     */
    private void setCurve(Actuator current, Node root) {
        ActuatorCurve acurve = (ActuatorCurve) current.getCurve();
        Element curve = m_document.createElement("curve");
        root.appendChild(curve);
        for (Key element : acurve.getKeys()) {
            setKey(element, curve);
        }
    }

    /**
     * Sets a timeline key.
     *
     * @param key Key
     * @param root Node
     */
    private void setKey(Key key, Node root) {
        Element node = m_document.createElement("key");
        node.setAttribute("type", key.getKeyType().toString());
        node.setAttribute("frame", Double.toString(key.getFrame()));
        node.setAttribute("value", Double.toString(key.getValue()));
        root.appendChild(node);
    }
}
