/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ZipConfig.java
 *
 * Created on November 23, 2004, 10:03
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ZipConfig.java,v 1.5 2005/01/25 23:55:12 jaquier Exp $
 */

package birg.bluemove.config.core;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.zip.*;

import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class implements the Config interface and handles <a
 * href="http://www.pkware.com/company/standards/appnote/">ZIP files</a>. This
 * class is a singleton because we handle only one archive at a given time (only
 * one project can be active at the same time).</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ZipConfig implements Config {

    static Logger logger = Logger.getLogger(ZipConfig.class);

    private String m_aName = null;
    private ZipFile m_file = null;

    static private ZipConfig m_instance;

    /**
     * Constructor. This constructor is private because ZipConfig implements a
     * singleton schema.
     */
    private ZipConfig() {
    }

    /**
     * Gets a new instance of ZipConfig or returns the current one. You must use
     * this method instead of <pre>ZipConfig xyz = new ZipConfig();</pre>.
     *
     * @return Config
     */
    static public Config getInstance() {
        if (m_instance == null) {
            m_instance = new ZipConfig();
        }
        return m_instance;
    }

    /**
     * Loads the ZIP archive into memory. Return false if the archive does not
     * exist or is not a valid ZIP file.
     *
     * @param aName String
     * @return boolean
     */
    public boolean loadArchive(String aName) {
        m_aName = aName;
        try {
            if (m_file != null) {
                m_file.close();
            }
            m_file = new ZipFile(aName);
        } catch (IOException e) {
            logger.error(e.getMessage());
            return false;
        }
        return true;
    }

    /**
     * Creates a new ZIP archive. As it is impossible to create an empty ZIP
     * file, we add a file named with the current date and time. So we know when
     * the project was created.
     *
     * @param aName String
     */
    public void createArchive(String aName) {
        // Format the current time. Use a "standard" name. See bug #0000003
        SimpleDateFormat formatter = new SimpleDateFormat(
                "'Created on 'dd MM yyyy - HH mm ss");
        Date currentTime = new Date();
        String dateString = formatter.format(currentTime);

        try {
            ZipOutputStream out = new ZipOutputStream(new FileOutputStream(
                    aName));
            out.putNextEntry(new ZipEntry(dateString));
            out.close();
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    /**
     * Reads a file from the ZIP archive. Returns null if the file does not
     * exist or is unreadable.
     *
     * @param fName String
     * @return InputStream
     */
    public InputStream readFromArchive(String fName) {
        InputStream cFileStream = null;
        ZipEntry zEntry = m_file.getEntry(fName);
        if (zEntry != null) {
            try {
                cFileStream = m_file.getInputStream(zEntry);
            } catch (IOException e) {
                logger.error(e.getMessage());
            }
        } else {
            logger.warn("Unable to read " + fName);
        }
        return cFileStream;
    }

    /**
     * Checks if a file exists in the ZIP archive.
     *
     * @param fName String
     * @return boolean
     */
    public boolean fileExists(String fName) {
        ZipEntry current = null;
        Enumeration zipEnum = m_file.entries();
        while (zipEnum.hasMoreElements()) {
            current = (ZipEntry) zipEnum.nextElement();
            if (fName.equals(current.getName())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Writes an outputstream to the ZIP file. Modification of an ZIP archive is
     * quite tricky in Java because it is... impossible. You need to create a
     * temporary archive, copy data from the original file to the temporary one
     * and then rename the temporary file to the original name.
     *
     * @param oS OutputStream
     * @param fName String
     * @throws IOException
     * @todo Splits this method into smaller ones.
     */
    public void writeToArchive(OutputStream oS, String fName) throws
            IOException {
        File zipFile = new File(m_aName);
        File tempZipFile = new File(m_aName + ".tmp");

        // Initialize a flag that will indicate that the jar was updated.
        boolean zipUpdated = false;

        try {
            // Create a temp jar file with no manifest. (The manifest will
            // be copied when the entries are copied.)
            ZipOutputStream tempZip =
                    new ZipOutputStream(new FileOutputStream(tempZipFile));

            tempZip.setLevel(Deflater.DEFAULT_COMPRESSION);

            // Allocate a buffer for reading entry data.
            byte[] buffer = null;
            int bytesRead;

            try {
                ZipEntry zEntry = new ZipEntry(fName);
                logger.debug("Adding: " + zEntry.getName());
                tempZip.putNextEntry(zEntry);
                buffer = ((ByteArrayOutputStream) oS).toByteArray();
                tempZip.write(buffer);
                oS.close();
                tempZip.closeEntry();

                // Loop through the jar entries and add them to the temp jar,
                // skipping the entry that was added to the temp jar already.
                Enumeration entries = m_file.entries();
                while (entries.hasMoreElements()) {
                    // Get the next entry.
                    ZipEntry entry = (ZipEntry) entries.nextElement();

                    // If the entry has not been added already, add it.
                    if (!entry.getName().equals(fName)) {
                        // Get an input stream for the entry.
                        InputStream entryStream = m_file.getInputStream(entry);

                        // Read the entry and write it to the temp jar.
                        logger.debug("Adding: " + entry.getName());
                        tempZip.putNextEntry(new ZipEntry(entry.getName()));
                        buffer = new byte[1024];
                        while ((bytesRead = entryStream.read(buffer)) != -1) {
                            tempZip.write(buffer, 0, bytesRead);
                        }
                        entryStream.close();
                        tempZip.closeEntry();
                    }

                }
                zipUpdated = true;
            } catch (Exception e) {
                logger.error(e.getMessage());
                // Add a stub entry here, so that the jar will close without an
                // exception.
                tempZip.putNextEntry(new ZipEntry("stub"));
            } finally {
                tempZip.close();
            }
        } finally {
            logger.debug(m_aName + " closed");

            // If the jar was not updated, delete the temp jar file.
            if (!zipUpdated) {
                tempZipFile.delete();
            }
        }

        // If the jar was updated, delete the original jar file and rename the
        // temp jar file to the original name.
        if (zipUpdated) {
            // Close the current archive. See bug #0000003
            m_file.close();
            zipFile.delete();
            tempZipFile.renameTo(zipFile);
            // Reload archive
            loadArchive(zipFile.getAbsolutePath());
            logger.info(m_aName + " updated");
        }
    }
}
