/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * PreferencesTree.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: PrefsTree.java,v 1.4 2005/01/27 22:03:13 jaquier Exp $
 */

package birg.bluemove.config.gui;

import java.awt.BorderLayout;
import java.util.Enumeration;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class displays a tree for user preferences. Each leaf of
 * the tree represents a different category of preferences. It is thus easier
 * for the user to find the given option he wants to modify.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class PrefsTree extends JPanel implements TreeSelectionListener {

    private DefaultTreeModel m_treeModel = null;
    DefaultMutableTreeNode m_top = new DefaultMutableTreeNode("Bluemove");

    private PrefsGUI m_prefsGui = null;

    /**
     * Constructor. Here we add the "preferences category" nodes.
     *
     * @param prefsGui PrefsGUI
     */
    public PrefsTree(PrefsGUI prefsGui) {
        m_prefsGui = prefsGui;
        m_treeModel = new DefaultTreeModel(m_top);
        jTree1.setModel(m_treeModel);

        // Build the prefs tree
        addNode("General", m_top);
        addNode("Player", m_top);

        expandAll(jTree1, true);

        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }


    /**
     * If expand is true, expands all nodes in the tree. Otherwise, collapses
     * all nodes in the tree.
     *
     * @param tree JTree
     * @param expand boolean
     */
    public void expandAll(JTree tree, boolean expand) {
        TreeNode root = (TreeNode) tree.getModel().getRoot();

        // Traverse tree from root
        expandAll(tree, new TreePath(root), expand);
    }

    /**
     * If expand is true, expands all nodes in the tree. Otherwise, collapses
     * all nodes in the tree.
     *
     * @param tree JTree
     * @param parent TreePath
     * @param expand boolean
     */
    private void expandAll(JTree tree, TreePath parent, boolean expand) {
        // Traverse children
        TreeNode node = (TreeNode) parent.getLastPathComponent();
        if (node.getChildCount() >= 0) {
            for (Enumeration e = node.children(); e.hasMoreElements(); ) {
                TreeNode n = (TreeNode) e.nextElement();
                TreePath path = parent.pathByAddingChild(n);
                expandAll(tree, path, expand);
            }
        }

        // Expansion or collapse must be done bottom-up
        if (expand) {
            tree.expandPath(parent);
        } else {
            tree.collapsePath(parent);
        }
    }


    /**
     * Adds a new node to the given root node.
     *
     * @param value String
     * @param node DefaultMutableTreeNode
     * @return DefaultMutableTreeNode
     */
    private DefaultMutableTreeNode addNode(String value,
                                           DefaultMutableTreeNode node) {
        DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(value);
        m_treeModel.insertNodeInto(newNode, node,
                                   m_treeModel.getChildCount(node));
        return newNode;
    }

    /**
     * Initializes the graphical components.
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        this.add(jScrollPane1, java.awt.BorderLayout.CENTER);
        jScrollPane1.getViewport().add(jTree1);
        jTree1.addTreeSelectionListener(this);
        jTree1.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    }

    JScrollPane jScrollPane1 = new JScrollPane();
    JTree jTree1 = new JTree();
    BorderLayout borderLayout1 = new BorderLayout();

    /**
     * The user selected a new node. Changes the preferences panel with the
     * appropriate panel.
     *
     * @param e TreeSelectionEvent
     */
    public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                      jTree1.getLastSelectedPathComponent();

        if (node == null) {
            return;
        }

        Object nodeInfo = node.getUserObject();
        m_prefsGui.setPrefsPanel(nodeInfo);
    }
}
