/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Main.java
 *
 * Created on 10. novembre 2004, 22:48
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Bluemove.java,v 1.17 2005/02/09 15:18:03 jaquier Exp $
 */

package birg.bluemove.main.core;

import java.io.File;
import java.util.prefs.Preferences;

import birg.bluemove.config.core.OpenProject;
import birg.bluemove.config.core.SaveProject;
import birg.bluemove.main.gui.BluemoveApp;
import birg.bluemove.module.core.ModulesManager;
import birg.bluemove.utils.InternetTools;
import birg.bluemove.utils.gui.SplashScreen;
import org.apache.log4j.*;


/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class is THE class. It creates the main objects needed
 * to start Bluemove.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class Bluemove {

    static Logger logger = Logger.getLogger(Bluemove.class);

    protected static String helpURL = "http://birg.epfl.ch/page56602.html";
    protected BluemoveApp m_app = null;
    protected File m_projectPath = null;
    protected ModulesManager m_mManager = ModulesManager.getInstance();
    Preferences m_prefs = null;

    /**
     * Constructor. Displays the splash screen and starts the Bluemove GUI.
     */
    public Bluemove() {
        initialize();
        logger.info("Starting Bluemove...");
        logger.debug("java.library.path=" +
                     System.getProperty("java.library.path"));
        logger.debug("os.name=" + System.getProperty("os.name"));

        m_prefs = Preferences.userNodeForPackage(this.getClass());
        // Setting system preferences
        if (m_prefs.getBoolean("aatext", false)) {
            // Must be called before swing stuff
            System.setProperty("swing.aatext", "true");
        }

        SplashScreen splash = new SplashScreen();
        splash.setLocationRelativeTo(null);
        splash.setVisible(true);

        m_app = new BluemoveApp(this);
        m_app.setSize(1024, 700);
        m_app.setLocationRelativeTo(m_app.getParent());
        m_app.setAppTitle("New project");
        m_app.setVisible(true);

        logger.info("Exiting Bluemove...");
    }

    /**
     * Returns the current project name.
     *
     * @return String
     */
    public String getProjectName() {
        return m_projectPath.getName();
    }

    /**
     * Creates a new project. In order to do that, clears the modules manager,
     * sets the current project to null and changes the window title.
     */
    public void newProject() {
        logger.info("You start a new project");
        m_mManager.clear();
        m_projectPath = null;
        m_app.setAppTitle("New project");
    }

    /**
     * Opens a project. After opening the project, set the <pre>lastDir</pre>
     * preference to the current project path.
     */
    public void openProject() {
        logger.info("You open a project");
        OpenProject oP = new OpenProject();
        oP.setCurrentDir(m_prefs.get("lastDir", ""));
        File f = oP.showDialog();
        if (f != null) {
            if (oP.openProjectFile(f)) {
                m_projectPath = f;
                this.addToProjectHistory(f);
                m_app.setAppTitle(f.getName());
            }
            m_prefs.put("lastDir", f.getParent());
        }
    }

    /**
     * Opens a project from history.
     */
    public void openProject(String fileName) {
        logger.info("You open a project");
        File f = new File(fileName);
        OpenProject oP = new OpenProject();
        if (f != null) {
            if (oP.openProjectFile(f)) {
                m_projectPath = f;
                m_app.setAppTitle(f.getName());
            }
        }
    }


    /**
     * Saves a project. If the project has never been saved, calls the
     * <pre>saveProjectAs()</pre> method.
     */
    public void saveProject() {
        logger.info("You save a project");
        SaveProject sP = new SaveProject();
        if (m_projectPath != null) {
            sP.saveProjectFile(m_projectPath, false);
        } else {
            saveProjectAs();
        }
    }

    /**
     * Saves a project as. Saves the current project to the given filename (from
     * file browser).
     */
    public void saveProjectAs() {
        logger.info("You save a project as...");
        SaveProject sP = new SaveProject();
        sP.setCurrentDir(m_prefs.get("lastDir", ""));
        File f = sP.showDialog();
        if (f != null) {
            if (sP.saveProjectFile(f, true)) {
                m_projectPath = f;
                this.addToProjectHistory(f);
                m_app.setAppTitle(f.getName());
            }
            m_prefs.put("lastDir", f.getParent());
        }
    }

    /**
     * Adds the project to the remember project list.
     *
     * @param f File
     */
    public void addToProjectHistory(File f) {
        m_prefs.put("projHist0", f.getAbsolutePath());
    }

    /**
     * Gets the last project from the history. Returns an array with the
     * absolute path and the filename of the project.
     *
     * @return String[][]
     */
    public String[][] getProjectHistory() {
        String temp[][] = new String[1][2];

        String s = m_prefs.get("projHist0", "");
        String tempPath[] = s.split(File.separator);
        temp[0][0] = tempPath[tempPath.length - 1];
        temp[0][1] = s;

        return temp;
    }

    /**
     * Displays the help. Opens an URL in the system browser. The URL should
     * point to a web page containing the Bluemove help.
     */
    public void displayHelp() {
        String browser = m_prefs.get("browser", "firefox");
        InternetTools.openURL(browser, helpURL);
    }

    /**
     * Initializes the important stuff such as log4j.
     */
    private void initialize() {
        BasicConfigurator.configure();
//        PropertyConfigurator.configure(resources.class.getResource(
//                "log4j.properties"));
        Logger.getRootLogger().setLevel(Level.INFO);
    }

    /**
     * This is the start of Bluemove. Simply creates a new Bluemove instance.
     *
     * @param args String[]
     */
    public static void main(String[] args) {
        new Bluemove();
    }

}
