/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Module.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Module.java,v 1.10 2005/01/25 23:47:13 jaquier Exp $
 */

package birg.bluemove.module.core;

import java.util.LinkedList;
import java.util.Observable;

import birg.bluemove.timelines.core.timeline.TimelineModule;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class represents a module unit. It contains components
 * (actuators, sensors, ...), has a given name and address. This class extends
 * <pre>Observable</pre> in order to notify registered objects that a module has
 * changed its state.</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class Module extends Observable {

    protected static Integer m_counter = 1;
    protected String m_name = null;
    protected String m_address = null;
    protected String m_comment = null;
    protected LinkedList<ModuleComponent> m_components = null;
    protected ModuleComponent m_lastRemoved = null;
    protected TimelineModule m_tlModule = null;
    protected ModulesManager m_mManager = ModulesManager.getInstance();

    /**
     * Constructor. Calls another constructor and increments the
     * <pre>m_counter</pre> value which is used to sets the module name.
     *
     * @throws Exception
     */
    public Module() throws Exception {
        this("Module" + m_counter, "42495247000" + m_counter);
        m_counter++;
    }

    /**
     * Constructor. Calls another constructor and increments the
     * <pre>m_counter</pre> value which is used to sets the module name.
     *
     * @throws Exception
     */
    public Module(String name) throws Exception {
        this(name, "42495247000" + m_counter);
        m_counter++;
    }

    /**
     * Constructor. Sets the name and address of the module. Creates the
     * components list and the module timeline.
     *
     * @param name String
     * @param address String
     * @throws Exception
     */
    public Module(String name, String address) throws Exception {
        setName(name);
        setAddress(address);
        m_components = new LinkedList<ModuleComponent>();
        m_tlModule = new TimelineModule();
    }

    /**
     * Returns the module timeline.
     *
     * @return TimelineModule
     */
    public TimelineModule getTimelineModule() {
        return m_tlModule;
    }

    /**
     * Sets the module timeline.
     *
     * @param tlModule TimelineModule
     */
    public void setTimelineModule(TimelineModule tlModule) {
        m_tlModule = tlModule;
    }

    /**
     * Gets the module name.
     *
     * @return String
     */
    public String getName() {
        return m_name;
    }

    /**
     * Sets the module name. Throws an exception if the given name already
     * exists for another module.
     *
     * @param name String
     * @throws Exception
     */
    public void setName(String name) throws Exception {
        Module temp = m_mManager.getModule(name);
        if (temp != null) {
            throw new Exception("Duplicate module name " + name);
        }
        m_name = name;
    }

    /**
     * Gets the module address.
     *
     * @return String
     */
    public String getAddress() {
        return m_address;
    }

    /**
     * Sets the module address.
     *
     * @param address String
     * @todo Maybe the module address should also be unique.
     */
    public void setAddress(String address) {
        m_address = address;
    }

    /**
     * Gets the comments for this module. It can be information about how to
     * configure it or anything else.
     *
     * @return String
     */
    public String getComment() {
        return m_comment;
    }

    /**
     * Sets the module comment.
     *
     * @param comment String
     */
    public void setComment(String comment) {
        m_comment = comment;
    }

    /**
     * Adds a component to the module. Notifies the observers that the module
     * state changed.
     *
     * @param component ModuleComponent
     */
    public void addComponent(ModuleComponent component) {
        m_components.add(component);
        setChanged();
        notifyObservers("add");
    }

    /**
     * Removes a given component. We do not really remove it but assign it to
     * the <pre>m_lastRemoved</pre> variable. This could be used to implement an
     * undo function. Notifies the observers that the module state changed.
     *
     * @param component ModuleComponent
     */
    public void removeComponent(ModuleComponent component) {
        m_lastRemoved = component;
        m_components.remove(component);
        setChanged();
        notifyObservers("remove");
    }

    /**
     * Gets the last component inserted in the module.
     *
     * @return ModuleComponent
     */
    public ModuleComponent getLastComponent() {
        return m_components.getLast();
    }

    /**
     * Gets the last removed component.
     *
     * @return ModuleComponent
     */
    public ModuleComponent getLastRemovedComponent() {
        return m_lastRemoved;
    }

    /**
     * Returns all the components of the module.
     *
     * @return LinkedList
     */
    public LinkedList<ModuleComponent> getComponentList() {
        return m_components;
    }

    /**
     * Gets the component with a given name. If no component found, return null.
     *
     * @param name String
     * @return ModuleComponent
     */
    public ModuleComponent getComponent(String name) {
        for (ModuleComponent element : m_components) {
            if (name.equals(element.getName())) {
                return element;
            }
        }
        return null;
    }

    /**
     * String representation of a module. Simply returns the module name.
     *
     * @return String
     */
    public String toString() {
        return m_name;
    }
}
