/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * ModuleManager.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModulesManager.java,v 1.6 2005/01/25 23:47:13 jaquier Exp $
 */

package birg.bluemove.module.core;

import java.awt.image.BufferedImage;
import java.util.LinkedList;
import java.util.Observable;

import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: This class manages all the modules. It is a singleton.
 * Everytime you need to access modules, you should use this class. From this
 * class, you can retrieve (almost) everything: modules, components, timelines,
 * curves, ...</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 * @author Cyril
 * @version 1.0
 */
public class ModulesManager extends Observable {

    static Logger logger = Logger.getLogger(ModulesManager.class);
    static private ModulesManager m_instance = null;

    private LinkedList<Module> m_modules = null;
    private BufferedImage m_buildImage = null;
    private String m_description = null;
    private Module m_lastRemoved = null;

    /**
     * Constructor. This constructor is private because
     * <pre>ModulesManager</pre> is a singleton.
     */
    private ModulesManager() {
        m_modules = new LinkedList<Module>();
    }

    /**
     * Returns the singleton instance. You must use this static method instead
     * of doing <pre>ModulesManager xyz = new ModulesManager();</pre>.
     *
     * @return ModulesManager
     */
    static public ModulesManager getInstance() {
        if (m_instance == null) {
            m_instance = new ModulesManager();
        }
        return m_instance;
    }

    /**
     * Adds a module to the manager. Notifies the observers that a module was
     * been added.
     *
     * @param module Module
     */
    public void addModule(Module module) {
        m_modules.add(module);
        setChanged();
        notifyObservers("add");
    }

    /**
     * Adds a list of modules. The list contains the modules name.
     *
     * @param mList LinkedList
     * @throws Exception
     */
    public void addModulesList(LinkedList<String> mList) throws Exception {
        for (String element : mList) {
            m_modules.add(new Module(element));
        }
    }

    /**
     * Remove a given module from the manager. We do not really remove it but
     * keep it into <pre>m_lastRemoved</pre>. Could be used to implement an undo
     * function.
     *
     * @param module Module
     */
    public void removeModule(Module module) {
        m_lastRemoved = module;
        m_modules.remove(module);
        setChanged();
        notifyObservers("remove");
    }

    /**
     * Gets a given module. Returns null if no module found.
     *
     * @param name String
     * @return Module
     */
    public Module getModule(String name) {
        for (Module element : m_modules) {
            if (name.equals(element.getName())) {
                return element;
            }
        }
        return null;
    }

    /**
     * Returns all the module that are in the manager.
     *
     * @return LinkedList
     */
    public LinkedList<Module> getModuleList() {
        return m_modules;
    }

    /**
     * Returns the last module added to the manager.
     *
     * @return Module
     */
    public Module getLastModule() {
        return m_modules.getLast();
    }

    /**
     * Gets the last module removed from the manager.
     *
     * @return Module
     */
    public Module getLastRemovedModule() {
        return m_lastRemoved;
    }

    /**
     * Sets the project image. This image should show to the user how to build
     * the modular robot.
     *
     * @param buildImage BufferedImage
     */
    public void setBuildImage(BufferedImage buildImage) {
        m_buildImage = buildImage;
    }

    /**
     * Gets the project image.
     *
     * @return BufferedImage
     */
    public BufferedImage getBuildImage() {
        return m_buildImage;
    }

    /**
     * Sets the project description. Should contains all information needed to
     * build the modular robot or any other information.
     *
     * @param description String
     */
    public void setDescription(String description) {
        m_description = description;
    }

    /**
     * Gets the project description.
     *
     * @return String
     */
    public String getDescription() {
        return m_description;
    }

    /**
     * Clears the modules manager. Clears the modules list, project image and
     * description. Notifies the observers that the manager state changed.
     */
    public void clear() {
        logger.debug("Clear");
        m_modules.clear();
        m_buildImage = null;
        m_description = null;
        setChanged();
        notifyObservers("clear");
    }
}
