/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Curve.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Curve.java,v 1.7 2005/01/26 22:41:43 drapel Exp $
 */

package birg.bluemove.timelines.core.curve;

import static java.lang.Math.*;
import java.util.Collections;
import java.util.Vector;

import birg.bluemove.module.core.ModuleComponent;
import birg.bluemove.timelines.core.interpolator.*;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: Curve.java,v 1.7 2005/01/26 22:41:43 drapel Exp $
 */

public class Curve {

    static Logger logger = Logger.getLogger(Curve.class);

    protected Vector<Key> m_keys = new Vector<Key>();
    protected double m_yAxisMin = 0.0;
    protected double m_yAxisMax = 0.0;

    protected static InterpolatorLinear intLinear = new InterpolatorLinear();
    protected static InterpolatorHermite intBezier = new InterpolatorHermite();

    protected ModuleComponent m_component = null;

    /**
     * Constructor.
     * @param component ModuleComponent
     */
    public Curve(ModuleComponent component) {
        m_component = component;
    }

    /**
     * Gets the component linked to this curve.
     * @return ModuleComponent
     */
    public ModuleComponent getModuleComponent() {
        return m_component;
    }

    /**
     * Sets the component linked to this curve.
     * @param component ModuleComponent
     */
    public void setModuleComponent(ModuleComponent component) {
        m_component = component;
    }

    /**
     * Adds a new key to this curve.
     * @param k Key
     */
    public void addKey(Key k) {
        m_keys.add(k);
        Collections.sort(m_keys);
        computeTangentsForSingleKey(k, 0.5);
    }

    /**
     * Removes the key from the curve.
     * @param k Key
     */
    public void removeKey(Key k) {
        m_keys.remove(k);
    }

    /**
     * Evaluates the curve at a given frame.
     * @param frame double
     * @return double
     */
    public double evaluate(double frame) {
        assert (m_keys.size() >= 2);

        Collections.sort(m_keys);
        if (frame < m_keys.firstElement().getFrame()) {
            return m_keys.firstElement().getValue();
        }

        if (frame >= m_keys.lastElement().getFrame()) {
            return m_keys.lastElement().getValue();
        }

        int n = 0;

        for (n = 0; n < m_keys.size() - 2; n++) {
            if (m_keys.get(n).getFrame() <= frame &&
                m_keys.get(n + 1).getFrame() > frame) {
                break;
            }
        }

        // start = previousKey index
        Key previousKey = m_keys.get(n);
        Key nextKey = m_keys.get(n + 1);

        // if and only if two linear keys -> linear interpolation
        // otherwise Bezier interpolation
        Vector<Key> tupleKeys = new Vector<Key>();
        tupleKeys.add(previousKey);
        tupleKeys.add(nextKey);

        if (previousKey.getKeyType().equals(Key.KeyType.LINEAR) &&
            nextKey.getKeyType().equals(Key.KeyType.LINEAR)) {
            return intLinear.evaluate(tupleKeys, frame);
        } else {
            return intBezier.evaluate(tupleKeys, frame);
        }

        //  assert (false);

        // return 0.0;
    }

    /**
     * Computes the tangents for all keys.
     * @param ease double
     */
    public void computeTangents(double ease) {
        int size = m_keys.size();

        for (int i = 0; i < size; i++) {
            computeTangentsForSingleKey(i, ease);
        }
    }

    /**
     * Computes the tangent for one key.
     * @param k Key
     * @param ease double
     */
    public void computeTangentsForSingleKey(Key k, double ease) {
        computeTangentsForSingleKey(m_keys.indexOf(k), ease);
    }

    /**
     * Computes the tangent for one key.
     * @param idx int
     * @param ease double
     */
    public void computeTangentsForSingleKey(int idx, double ease) {
        Key nextKey = null;
        Key previousKey = null;
        if (idx + 1 < m_keys.size()) {
            nextKey = m_keys.get(idx + 1);
        }
        if (idx - 1 >= 0) {
            previousKey = m_keys.get(idx - 1);
        }

        m_keys.get(idx).computeTangents(ease, previousKey, nextKey);
    }


    /**
     * Finds first occurence of a key in the vector with the
     * same frame value as specified in the argument.
     *
     * @param frame value
     * @return A key if a matching frame is found, null otherwise
     */
    public Key searchKeyAtFrame(double frame) {
        for (Key k : m_keys) {
            if (abs(k.getFrame() - frame) < Double.MIN_VALUE) {
                return k;
            }
        }
        return null;
    }


    /**
     * Gets a Vector containing all keys.
     * @return Vector
     */
    public Vector<Key> getKeys() {
        return m_keys;
    }

    /**
     * Gets the amount of keys in this curve.
     * @return int
     */
    public int getNbKeys() {
        return m_keys.size();
    }
}
