/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * InterpolatorBezier.java
 *
 * Created on 10. novembre 2004, 23:16
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: InterpolatorHermite.java,v 1.6 2005/01/26 22:41:43 drapel Exp $
 */

package birg.bluemove.timelines.core.interpolator;

import java.util.List;

import birg.bluemove.utils.V2D;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: InterpolatorHermite.java,v 1.6 2005/01/26 22:41:43 drapel Exp $
 */
public class InterpolatorHermite implements Interpolator {

    protected int m_bissectionSteps = 16;

    /** Creates a new instance of InterpolatorSpline */
    public InterpolatorHermite() {
    }

    /**
     * Evaluates a 1D component on a Hermite curve.
     * @param t time parameter
     * @param ptStart position of start point
     * @param ptEnd position of end point
     * @param tgStart tangent of start point
     * @param tgEnd tangent of end point
     * @return double
     */
    protected double hermitePoint(double t, double ptStart, double ptEnd,
                                  double tgStart,
                                  double tgEnd) {
        // compute Hermite spline polynomials
        double h1p = 2.0 * t * t * t - 3.0 * t * t + 1.0;
        double h2p = -(2.0 * t * t * t) + 3.0 * t * t;

        double h1t = t * t * t - (2.0 * t * t) + t;
        double h2t = t * t * t - (t * t);

        return ptStart * h1p + tgStart * h1t + tgEnd * h2t + ptEnd * h2p;
        //return ptStart * h1p + ptEnd * h2p;

    }

    /**
     * Evaluates y=f(x) with x = frame and y is the value on the spline.
     * Use the bissection method to solve this problem (the curve has
     * an additionnal constraint, it must be a function).
     * @param keys list with exactly two keys (start/end)
     * @param frame "x" value in f(x)
     * @return double "y" value
     */
    public double evaluate(List<Key> keys, double frame) {
        assert (keys.size() == 2);

        // get reference to linear keys
        Key k1 = keys.get(0);
        Key k2 = keys.get(1);

        // frames numbers must be consistent
        assert (k1.getFrame() < k2.getFrame());

        // get coordinates of points in Bezier hull
        V2D ptStart = k1.getCoordinates();
        V2D ptEnd = k2.getCoordinates();
        // System.err.println(ptEnd);
        V2D ptCtrl1 = k1.getTangentOutput();
        V2D ptCtrl2 = k2.getTangentInput();

        // find 't' on curve for a given 'x' (frame)
        // bissection method ! the genuine one !

        double t1 = 0.0;
        double t2 = 1.0;

        double t = (t1 + t2) * 0.5;

        // recursively subdivide the region of interest where the solution
        // is located
        double x = 0.0;

        for (int i = 0; i < m_bissectionSteps; i++) {
            x = hermitePoint(t, ptStart.x, ptEnd.x, ptCtrl1.x, ptCtrl2.x);
            if (x < frame) {
                t1 = t;
            } else {
                t2 = t;
            }
            t = (t1 + t2) * 0.5;
        }
        return hermitePoint(t, ptStart.y, ptEnd.y, ptCtrl1.y, ptCtrl2.y);
    }

    /**
     * Sets the bissection steps (more steps = more accuracy in the curve
     * but slower computation). Values between 5 and 15 are a good tradeoff.
     * @param steps int number of steps
     */
    public void setBissectionSteps(int steps) {
        assert (steps > 0); m_bissectionSteps = steps;
    }

    /**
     * Gets the number of bissection steps.
     * @return int number of steps
     */
    public int getBissectionSteps() {
        return m_bissectionSteps;
    }

}
