/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ |/ / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Key.java
 *
 * Created on 10. novembre 2004, 23:16
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Key.java,v 1.6 2005/02/03 13:42:12 jaquier Exp $
 */

package birg.bluemove.timelines.core.interpolator;

import birg.bluemove.utils.V2D;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: Key.java,v 1.6 2005/02/03 13:42:12 jaquier Exp $
 */
public class Key implements Comparable {

    public enum KeyType {
        LINEAR, HERMITE, HERMITE_CORNER, UNDEFINED
    }


    protected double m_value = 0.0;
    protected double m_frame = 0.0;

    protected V2D m_inputTangent = new V2D(0, 0);
    protected V2D m_outputTangent = new V2D(0, 0);

    protected KeyType m_keyType = KeyType.UNDEFINED;

    /**
     * Creates a new instance of Key.
     * @param keyType KeyType
     * @param frame frame position (x)
     * @param value vertical position (y)
     */
    public Key(KeyType keyType, double frame, double value) {
        assert (keyType != KeyType.UNDEFINED);
        m_keyType = keyType;
        setValue(value);
        setFrame(frame);
    }

    /**
     * Gets key type.
     * @return KeyType
     */
    public KeyType getKeyType() {
        return m_keyType;
    }

    /**
     * Sets key type.
     * @param keyType KeyType
     */
    public void setKeyType(KeyType keyType) {
        m_keyType = keyType;
    }


    /**
     * Sets value (y).
     * @param value double
     */
    public void setValue(double value) {
        m_value = value;
    }

    /**
     * Gets value (y).
     * @return double y
     */
    public double getValue() {
        return m_value;
    }

    /**
     * Sets frame (x).
     * @param frame double
     */
    public void setFrame(double frame) {
        m_frame = frame;
    }

    /**
     * Gets frame (x).
     * @return double x
     */
    public double getFrame() {
        return m_frame;
    }

    /**
     * Gets coordinates (x,y) in a V2D.
     * @return V2D
     */
    public V2D getCoordinates() {
        return new V2D(m_frame, m_value);
    }

    /**
     * Sets coordinates from a V2D.
     * @param v V2D
     */
    public void setCoordinates(V2D v) {
        m_frame = v.x;
        m_value = v.y;
    }

    /**
     * CompareTo implementation. The paramater used for comparison is
     * the frame value.
     * @param o Object
     * @return int
     */

    public int compareTo(java.lang.Object o) {
        if (!(o instanceof Key)) {
            throw new ClassCastException("A key object is expected.");
        }

        Key anotherKey = (Key) o;

        if (m_frame < anotherKey.getFrame()) {
            return -1;
        }
        if (Math.abs(m_frame - anotherKey.getFrame()) < Double.MIN_VALUE) {
            return 0;
        }
        return 1;
    }


    /**
     * Sets the input tangent. This vector represents the "arriving" velocity
     * at that point (in a GUI, this vector is inverted, be careful on that
     * point)
     * @param tg V2D
     */
    public void setTangentInput(V2D tg) {
        m_inputTangent = tg;
    }

    /**
     * Sets the output tangent. This vector represents the "outgoing" velocity
     * at that point.
     * @param tg V2D
     */
    public void setTangentOutput(V2D tg) {
        m_outputTangent = tg;
    }

    /**
     * Gets a copy of the input tangent (not the reference)
     * Use setTangentInput if you want to modify the tangent.
     * @return V2D a copy of the tangent
     */
    public V2D getTangentInput() {
        return (V2D) m_inputTangent.clone();
    }

    /**
     * Get a copy of the output tangent (not the reference)
     * Use setTangentOutput if you want to modify the tangent.
     * @return V2D
     */
    public V2D getTangentOutput() {
        return (V2D) m_outputTangent.clone();
    }

    /**
     * Computes the tangents using the usual Hermite tangents formula.
     * Clip them to keep a reasonable length. The horizontal component
     * won't exceed the position of the previous or next key.
     *
     * @param ease double between 0.0 and 1.0 for a normal use
     * @param previousKey Key in front of the current key
     * @param nextKey Key next to the current key
     */
    public void computeTangents(double ease, Key previousKey, Key nextKey) {
        // degenerated case, we don't handle it (tangent = zero).
        if (previousKey == null && nextKey == null) {
            return;
        }

        V2D p1 = null, p2 = null;

        // if previousKey doesn't exist, only use the nextKey
        if (previousKey == null) {
            p1 = getCoordinates();
        } else {
            p1 = previousKey.getCoordinates();
        }

        // if nextKey doesn't exist, only use the previousKey
        if (nextKey == null) {
            p2 = getCoordinates();
        } else {
            p2 = nextKey.getCoordinates();
        }

        // find tangent vector
        V2D tg = V2D.sub(p2, p1);

        // scale tangent vector (to have a curve behaving like a function)
        double length1 = Double.MAX_VALUE;
        double length2 = Double.MAX_VALUE;

        // compute distance between mid-key and previous key
        if (previousKey != null) {
            length1 = (double) (previousKey.getFrame() - getFrame());
            length1 = Math.abs(length1);
            if (length1 <= Double.MIN_VALUE) {
                length1 = 1.0;
            }
        }

        // compute distance between mid-key and next key
        if (nextKey != null) {
            length2 = (double) (nextKey.getFrame() - getFrame());
            length2 = Math.abs(length2);

            if (length2 <= Double.MIN_VALUE) {
                length2 = 1.0;
            }
        }

        tg = V2D.normalize(tg);
        setTangentInput(V2D.mul(tg, length1));
        setTangentOutput(V2D.mul(tg, length2));
    }

    /**
     * Prints the key as a string.
     * @return String
     */
    public String toString() {
        String s = "[" + m_frame + ";" + m_value + "]";
        return s;
    }
}
