/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * Player.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: Player.java,v 1.3 2005/01/27 16:32:34 jaquier Exp $
 */

package birg.bluemove.timelines.core.timeline;

import java.util.HashMap;
import java.util.Iterator;

import birg.bluemove.communication.bluetooth.Bluetooth;
import birg.bluemove.communication.core.*;
import birg.bluemove.module.core.*;
import birg.bluemove.timelines.core.curve.ActuatorCurve;
import birg.bluemove.timelines.core.curve.Curve;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class Player extends Thread {

    private Double m_cTime = 0.0;
    private TimelinesManager m_tlManager = null;
    private ModulesManager m_mManager = null;
    private ModuleDataManager m_dataManager = null;
    private Boolean m_isActive = true;
    private DataInterface m_bluetooth = null;

    /**
     * Constructor.
     * @param tlManager TimelinesManager
     */
    public Player(TimelinesManager tlManager) throws Exception {
        m_tlManager = tlManager;
        m_mManager = ModulesManager.getInstance();
        m_dataManager = ModuleDataManager.getInstance();
        // Perform Bluetooth tests
        m_bluetooth = Bluetooth.getInstance();
    }

    public boolean getPlayingState() {
        return m_isActive;
    }

    /**
     * Sets playing state.
     */
    public void setPlayingState(boolean value) {
        m_isActive = value;
    }

    public double getCurrentTime() {
        return m_cTime;
    }

    public void setCurrentTime(double value) {
        m_cTime = value;
    }

    /**
     * Creates a HashMap with active modules only.
     * @return HashMap
     */
    private HashMap<ActuatorCurve, ActuatorData> filterModules() {
        // Take only the active component
        HashMap<ActuatorCurve, ActuatorData> temp = null;
        temp = new HashMap<ActuatorCurve, ActuatorData>();
        for (Module element : m_mManager.getModuleList()) {
            TimelineModule tlM = element.getTimelineModule();
            if (tlM.isActive()) {
                ModuleData m = new ModuleData(element.getAddress());
                m_dataManager.addModule(m);
                for (ModuleComponent comp : element.getComponentList()) {
                    Curve curve = comp.getCurve();
                    if (curve instanceof ActuatorCurve) {
                        ActuatorCurve acurve = (ActuatorCurve) curve;
                        if (acurve.isActive()) {
                            ActuatorData s = new ActuatorData(comp.getName());
                            m.addComponent(s);
                            temp.put(acurve, s);
                        }
                    }
                }
            }
        }
        return temp;
    }

    /**
     * Plays and sends data from timelines.
     */
    public void run() {
        // Take only the active component
        HashMap<ActuatorCurve, ActuatorData> aList = filterModules();

        m_bluetooth.startConnections(m_dataManager);

        // Playing
        double endTime = m_tlManager.getEndTime();
        double increment = m_tlManager.getIncrement();
        long sleepTime = m_tlManager.getSleepTime();
        while (getPlayingState()) {
            Iterator iter = aList.keySet().iterator();
            while (iter.hasNext()) {
                ActuatorCurve curve = (ActuatorCurve) iter.next();
                if (curve.getNbKeys() > 1) {
                    ActuatorData data = aList.get(curve);
                    double value = curve.evaluateRealValue(m_cTime);
                    data.setValue(value);
                }
            }

            m_bluetooth.sendData(m_dataManager);

            m_cTime += increment;
            if (m_cTime > endTime) {
                if (m_tlManager.getRepeat()) {
                    setCurrentTime(0.0);
                } else {
                    setPlayingState(false);
                    m_cTime -= increment;
                }
            }

            try {
                sleep(sleepTime);
            } catch (InterruptedException e) {
                System.err.println(e.getMessage());
            }
        }

        // Sauvegarde le temps courant
        m_tlManager.setCurrentTime(m_cTime);
        m_dataManager.clear();
        m_bluetooth.stopConnections();
    }
}
