/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * TimelinesTab.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleInfoNode.java,v 1.11 2005/02/09 15:18:03 jaquier Exp $
 */


package birg.bluemove.timelines.gui.infotree;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.*;

import javax.swing.ImageIcon;

import birg.bluemove.module.core.*;
import birg.bluemove.resources.resources;
import birg.bluemove.timelines.gui.JTimelineComponent;
import birg.bluemove.timelines.gui.infotree.nodes.*;
import birg.bluemove.timelines.gui.timelinestree.TimelinesTree;
import birg.bluemove.utils.gui.FontTools;
import birg.bluemove.utils.gui.GUITools;
import org.apache.log4j.Logger;

/**
 *
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG / EPFL</p>
 *
 * @author Kevin Drapel
 * @version $Id: ModuleInfoNode.java,v 1.11 2005/02/09 15:18:03 jaquier Exp $
 */

public class ModuleInfoNode extends JTimelineComponent implements MouseListener,
        Observer {

    static Logger logger = Logger.getLogger(ModuleInfoNode.class);
    Module m_module = null;

    Vector<ModuleComponentInfoNode> m_componentInfoNodes = null;

    HashMap<String, Rectangle> m_activationRect;

    Color m_colorIsActive = new Color(0.75f, 1.0f, 0.7f);
    Color m_colorIsInactive = new Color(1.0f, 0.75f, 0.4f);

    /**
     * Constructor.
     * @param module Module
     */
    public ModuleInfoNode(Module module) {
        this.setLayout(null);

        m_module = module;
        m_module.addObserver(this);
        m_componentInfoNodes = new Vector<ModuleComponentInfoNode>();

        // hashmap for activation regions (icons)
        m_activationRect = new HashMap<String, Rectangle>();
        this.addMouseListener(this);
        this.setToolTipText(m_module.getComment());
    }

    /**
     * Gets module linked to this node.
     * @return Module
     */
    public Module getModule() {
        return m_module;
    }

    /**
     * Adds a component to this module.
     * @param component ModuleComponent
     */
    public void addComponent(ModuleComponent component) {
        ModuleComponentInfoNode m = null;

        if (component instanceof Actuator) {
            m = new ActuatorInfoNode(component);
        }

        if (component instanceof Sensor) {
            m = new SensorInfoNode(component);
        }

        assert (m != null);

        m_componentInfoNodes.add(m);
        this.add(m);
    }

    /**
     * Removes a component from this module.
     * @param component ModuleComponent
     */
    public void removeComponent(ModuleComponent component) {
        for (ModuleComponentInfoNode element : m_componentInfoNodes) {
            if (element.getComponent() == component) {
                m_componentInfoNodes.remove(element);
                this.remove(element);
                break;
            }
        }
    }


    /**
     * Paints the border.
     * @param g Graphics
     */

    public void paintBorder(Graphics g) {
        Graphics2D g2d = (Graphics2D) g.create();
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        g2d.setColor(Color.BLACK);
        g2d.drawRect(bounds.x, bounds.y, bounds.width - 1, bounds.height);
        g2d.dispose();
    }


    /**
     * Paints the component.
     * @param g Graphics
     */

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g.create();
        g2d.setRenderingHints(GUITools.getBestQualityHints());

        //Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getClosedHeight());

        GradientPaint gradient;

        if (m_module.getTimelineModule().isActive()) {
            gradient = new GradientPaint(0, 0, m_colorIsActive,
                                         0, this.getClosedHeight(), Color.WHITE,
                                         false);

        } else {
            gradient = new GradientPaint(0, 0, m_colorIsInactive,
                                         0, this.getClosedHeight(), Color.WHITE,
                                         false);

        }

        g2d.setPaint(gradient);
        g2d.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

        // draw icons and store the activation rectangles in a hashmap
        int yIcon = (getClosedHeight() - 16) / 2;

        Rectangle r1 = GUITools.drawIcon(g2d, getIconLed(), 5, yIcon);
        Rectangle r2 = GUITools.drawIcon(g2d, getIconArrow(), 5 + 5 + 16, yIcon);
        m_activationRect.put("Icon.Enabled", r1);
        m_activationRect.put("Icon.Expanded", r2);

        Font font = FontTools.getControlFontBold();
        int fontHeight = FontTools.getMaxHeight(font);
        int yString = fontHeight + (getClosedHeight() - fontHeight) / 2;

        g2d.setColor(Color.BLACK);
        g2d.setFont(font);
        g2d.drawString(m_module.getName(), 5 + 5 + 16 + 16 + 10, yString);

        g2d.setFont(FontTools.getControlFontItalic());

        GUITools.drawIcon(g2d, GUITools.getImageIcon("icon_address.png"),
                          5 + 5 + 16 + 16 + 16 + 100,
                          yIcon);

        g2d.drawString(m_module.getAddress(), 5 + 5 + 16 + 16 + 16 + 116 + 10,
                       yString);

        g2d.dispose();

    }


    /**
     * Gets LED icon.
     * @return ImageIcon
     */
    protected ImageIcon getIconLed() {
        if (m_module.getTimelineModule().isActive()) {
            return new ImageIcon(resources.class.getResource(
                    "icon_greenlight.png"));
        } else {
            return new ImageIcon(resources.class.getResource(
                    "icon_redlight.png"));
        }
    }

    /**
     * Gets arrow icon.
     * @return ImageIcon
     */
    protected ImageIcon getIconArrow() {
        if (isExpanded()) {
            return new ImageIcon(resources.class.getResource(
                    "icon_arrowdown.png"));
        } else {
            return new ImageIcon(resources.class.getResource(
                    "icon_arrowright.png"));
        }
    }

    public Vector<ModuleComponentInfoNode> getComponentInfoNodes() {
        return m_componentInfoNodes;
    }


    /**
     * Mouse clicked event.
     * @param mouseEvent MouseEvent
     */
    public void mouseClicked(MouseEvent mouseEvent) {
        Rectangle r1 = m_activationRect.get("Icon.Enabled");

        if (r1 != null) {
            Point p = mouseEvent.getPoint();

            if (r1.contains(p)) {
                m_module.getTimelineModule().setActive(!m_module.
                        getTimelineModule().isActive());
                synchronizeOpposite();
                getOpposite().repaint();
                repaint();
                return;
            }
        }

        Rectangle r2 = m_activationRect.get("Icon.Expanded");

        if (r2 != null) {
            Point p = mouseEvent.getPoint();

            if (r2.contains(p)) {
                if (m_componentInfoNodes.size() == 0) {
                    setExpanded(false);
                } else {
                    setExpanded(!isExpanded());
                }

                updateExpandedState();

                ModulesTree mt = (ModulesTree)this.getParent();
                mt.refreshModulesLocation();
                mt.repaint();

                synchronizeOpposite();
                TimelinesTree mt2 = (TimelinesTree) getOpposite().getParent();
                mt2.refreshModulesLocation();
                mt2.repaint();

                return;
            }
        }
    }

    /**
     * Gets expanded height for this module.
     * @return int
     */
    public int getExpandedHeight() {
        int h = this.getClosedHeight();
        for (Component c : this.getComponents()) {
            h += c.getHeight();
        }
        return h;
    }

    /**
     * Refresh all components location and repaint.
     */
    public void refreshComponentsLocation() {
        int y = getClosedHeight();

        for (Component c : this.getComponents()) {
            c.setLocation(0, y);
            y += c.getHeight();
        }

        updateExpandedState();

        ModulesTree mt = (ModulesTree)this.getParent();
        mt.refreshModulesLocation();
        mt.invalidate();
        mt.repaint();
    }

    /**
     * Resets the size and location of components.
     */
    public void resetSizeLocation() {

        int y = getClosedHeight();

        for (Component c : this.getComponents()) {
            ModuleComponentInfoNode mcin = (ModuleComponentInfoNode) c;
            c.setLocation(0, y);
            if (mcin.isExpanded()) {
                int height = mcin.getHeight();
                mcin.setCurrentDimension(getWidth(), height);
                y += height;
            } else {
                int height = getClosedHeight();
                mcin.setCurrentDimension(getWidth(), height);
                y += height;
            }
        }
    }


    public void mousePressed(MouseEvent mouseEvent) {
    }

    public void mouseReleased(MouseEvent mouseEvent) {
    }

    public void mouseEntered(MouseEvent mouseEvent) {
    }

    public void mouseExited(MouseEvent mouseEvent) {
    }

    /**
     * Observer for updates in the module manager.
     * @param o Observable
     * @param arg Object
     */
    public void update(Observable o, Object arg) {
        logger.debug("Update observer --> " + this.getClass().getName());
        if (o instanceof Module) {
            if (arg.toString().equals("add")) {
                ModuleComponent c = m_module.getLastComponent();
                this.addComponent(c);
            } else if (arg.toString().equals("remove")) {
                ModuleComponent c = m_module.getLastRemovedComponent();
                this.removeComponent(c);
            }
        }
    }
}
