/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * TimelinesTab.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModulesTree.java,v 1.12 2005/02/09 15:18:03 jaquier Exp $
 */


package birg.bluemove.timelines.gui.infotree;


import java.awt.Component;
import java.awt.Font;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import birg.bluemove.module.core.Module;
import birg.bluemove.module.core.ModulesManager;
import birg.bluemove.timelines.gui.TimelinesTab;
import birg.bluemove.timelines.gui.infotree.nodes.ModuleComponentInfoNode;
import birg.bluemove.timelines.gui.timelinestree.TimelinesTree;
import birg.bluemove.utils.gui.FontTools;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: ModulesTree.java,v 1.12 2005/02/09 15:18:03 jaquier Exp $
 */

public class ModulesTree extends JComponent implements AncestorListener {

    ModulesManager m_modulesManager = null;
    Vector<ModuleInfoNode> m_modulesInfoNodes = new Vector<ModuleInfoNode>();

    protected int m_verticalOffset = 0;
    protected TimelinesTab m_timelinesTab = null;
    protected TimelinesTree m_timelinesTree;

    /**
     * Constructor.
     * @param modulesManager ModulesManager
     */
    public ModulesTree(ModulesManager modulesManager, TimelinesTab tab) {
        m_modulesManager = modulesManager;
        m_timelinesTab = tab;
        this.addAncestorListener(this);
    }

    /**
     * Sets the vertical offset. Used for scrolling the timelines.
     * @param offset int
     */
    public void setVerticalOffset(int offset) {
        m_verticalOffset = offset;
    }

    /**
     * Gets the vertical offset used for scrolling.
     * @return int
     */
    public int getVerticalOffset() {
        return m_verticalOffset;
    }

    /**
     * Resets size and location of modules.
     */
    public void resetSizeLocation() {
        Font font = FontTools.getControlFont();
        int height = FontTools.getMaxHeight(font);
        int y = m_verticalOffset;

        for (Component c : this.getComponents()) {
            ModuleInfoNode min = (ModuleInfoNode) c;
            c.setLocation(0, y);
            if (min.isExpanded()) {
                int h = c.getHeight();

                c.setSize(getWidth(), h);
                y += h;
            } else {
                c.setSize(getWidth(), height * 3);
                y += height * 3;
            }
        }

        m_timelinesTree.resetSizeLocation();
        m_timelinesTab.setScrollbarRange();
    }

    /**
     * Refresh location of modules.
     */
    public void refreshModulesLocation() {
        int y = m_verticalOffset;

        for (Component c : this.getComponents()) {
            c.setLocation(0, y);
            y += c.getHeight();
        }

        m_timelinesTree.refreshModulesLocation();
        m_timelinesTab.setScrollbarRange();
    }

    /**
     * Gets current height with all modules and their components.
     * @return int
     */
    public int getCurrentHeight() {
        int h = 0;

        for (Component c : this.getComponents()) {
            h += c.getHeight();
        }
        return h;
    }

    /**
     * Adds a new module.
     * @param module Module
     */
    public void addModule(Module module) {
        ModuleInfoNode m = new ModuleInfoNode(module);
        m_modulesInfoNodes.add(m);
        this.add(m);
    }

    /**
     * Removes a module.
     * @param module Module
     */
    public void removeModule(Module module) {
        for (ModuleInfoNode element : m_modulesInfoNodes) {
            if (element.getModule() == module) {
                m_modulesInfoNodes.remove(element);
                this.remove(element);
                break;
            }
        }
    }

    /**
     * Repaints modules and their components.
     */
    public void repaintComponents() {
        for (ModuleInfoNode element : m_modulesInfoNodes) {
            for (ModuleComponentInfoNode comp :
                 element.getComponentInfoNodes()) {
                comp.updateExpandedState();
                comp.repaint();
            }
        }
    }

    /**
     * Remove all modules.
     */
    public void clear() {
        for (ModuleInfoNode element : m_modulesInfoNodes) {
            this.remove(element);
        }
        m_modulesInfoNodes.clear();
    }

    /**
     * Gets all modules nodes.
     * @return Vector
     */
    public Vector<ModuleInfoNode> getNodes() {
        return m_modulesInfoNodes;
    }

    public void setTimelinesTree(TimelinesTree tree) {
        m_timelinesTree = tree;
    }

    /**
     * Ancestor added event.
     * @param ancestorEvent AncestorEvent
     */
    public void ancestorAdded(AncestorEvent ancestorEvent) {
        resetSizeAll();
    }

    public void resetSizeAll() {
        resetSizeLocation();
        for (Component c : this.getComponents()) {
            ((ModuleInfoNode) c).resetSizeLocation();
        }
    }

    /**
     * Ancestor removed event.
     * @param ancestorEvent AncestorEvent
     */
    public void ancestorRemoved(AncestorEvent ancestorEvent) {
    }

    /**
     * Ancestor moved event.
     * @param ancestorEvent AncestorEvent
     */
    public void ancestorMoved(AncestorEvent ancestorEvent) {
    }

}
