package birg.bluemove.timelines.gui.infotree.nodes;


import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.HashMap;

import javax.swing.ImageIcon;

import birg.bluemove.module.core.Actuator;
import birg.bluemove.module.core.ModuleComponent;
import birg.bluemove.resources.resources;
import birg.bluemove.timelines.core.curve.ActuatorCurve;
import birg.bluemove.timelines.gui.infotree.ModuleInfoNode;
import birg.bluemove.timelines.gui.timelinestree.ModuleTimelineNode;
import birg.bluemove.utils.gui.FontTools;
import birg.bluemove.utils.gui.GUITools;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class ActuatorInfoNode extends ModuleComponentInfoNode {

    HashMap<String, Rectangle> m_activationRect;

    boolean resizingDrag = false;

    Color m_colorIsActive = new Color(0.90f, 1.0f, 0.9f);
    Color m_colorIsInactive = new Color(1.0f, 0.90f, 0.6f);

    ActuatorCurve m_actuatorCurve = null;

    /**
     * Constructor.
     * @param moduleComponent ModuleComponent
     */
    public ActuatorInfoNode(ModuleComponent moduleComponent) {
        super(moduleComponent);
        // hashmap for activation regions (icons)
        m_activationRect = new HashMap<String, Rectangle>();

        //  assert (moduleComponent instanceof Actuator);
        m_actuatorCurve = (ActuatorCurve) m_moduleComponent.getCurve();

    }

    /**
     * Draws the border.
     * @param g Graphics
     */
    public void paintBorder(Graphics g) {
        //super.paintBorder(g);
        Graphics2D g2d = (Graphics2D) g.create();
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        g2d.setColor(Color.BLACK);
        g2d.drawRect(bounds.x, bounds.y, bounds.width - 1, bounds.height);
        g2d.dispose();
    }

    /**
     * Paints an actuator info node.
     * @param g Graphics
     */
    public void paintComponent(Graphics g) {
        //  super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g.create();
        g2d.setRenderingHints(GUITools.getBestQualityHints());

        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        GradientPaint gradient;

        if (m_actuatorCurve.isActive()) {
            gradient = new GradientPaint(0, 0, m_colorIsActive,
                                         0, this.getClosedHeight() * 2,
                                         Color.WHITE,
                                         false);

        } else {
            gradient = new GradientPaint(0, 0, m_colorIsInactive,
                                         0, this.getClosedHeight() * 2,
                                         Color.WHITE,
                                         false);

        }

        g2d.setPaint(gradient);
        g2d.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

        if (isExpanded()) {
            g2d.setColor(new Color(0.8f, 0.8f, 0.8f));
            g2d.fillRect(bounds.x, bounds.y + bounds.height - 7, bounds.width,
                         7);
        }
        // draw icons and store the activation rectangles in a hashmap
        int yIcon = (getClosedHeight() - 16) / 2;
        Rectangle r1 = GUITools.drawIcon(g2d, getIconLed(), 5 + 16, yIcon);
        Rectangle r2 = GUITools.drawIcon(g2d, getIconArrow(), 5 + 5 + 16 + 16,
                                         yIcon);

        m_activationRect.put("Icon.Enabled", r1);
        m_activationRect.put("Icon.Expanded", r2);

        Font font = FontTools.getControlFont();
        int fontHeight = FontTools.getMaxHeight(font);

        int yString = fontHeight + (getClosedHeight() - fontHeight) / 2;

        g2d.setColor(Color.BLACK);
        g2d.drawString(m_moduleComponent.getName(), 5 + 5 + 16 + 16 + 16 + 10,
                       yString);

        g2d.setFont(FontTools.getControlFontItalic());

        GUITools.drawIcon(g2d, getIconComponent(), 5 + 5 + 16 + 16 + 16 + 100,
                          yIcon);

        g2d.drawString(getDescriptionComponent(),
                       5 + 5 + 16 + 16 + 16 + 116 + 10, yString);

        g2d.dispose();
    }

    /**
     * Gets icons used by this component.
     * @return ImageIcon
     */
    protected ImageIcon getIconComponent() {
        if (m_moduleComponent instanceof Actuator) {
            return GUITools.getImageIcon("icon_actuator.png");
        } else {
            return GUITools.getImageIcon("icon_sensor.png");
        }
    }

    /**
     * Creates a summary of the module parameters.
     * @return String
     */
    protected String getDescriptionComponent() {
        if (m_moduleComponent instanceof Actuator) {
            Actuator actuator = (Actuator) m_moduleComponent;
            double min = actuator.getRangeMin();
            double max = actuator.getRangeMax();
            return "[" + Double.toString(min) + ";" + Double.toString(max) +
                    "]";
        } else {
            return "sensor";
        }
    }


    /**
     * Gets the LED icon
     * @return ImageIcon
     */
    protected ImageIcon getIconLed() {
        if (m_actuatorCurve.isActive()) {
            return new ImageIcon(resources.class.getResource(
                    "icon_greenlight.png"));
        } else {
            return new ImageIcon(resources.class.getResource(
                    "icon_redlight.png"));
        }
    }

    /**
     * Gets the arrow icon.
     * @return ImageIcon
     */
    protected ImageIcon getIconArrow() {
        if (isExpanded()) {
            return new ImageIcon(resources.class.getResource(
                    "icon_arrowdown.png"));
        } else {
            return new ImageIcon(resources.class.getResource(
                    "icon_arrowright.png"));
        }
    }

    /**
     * Mouse pressed event.
     * @param mouseEvent MouseEvent
     */
    public void mousePressed(MouseEvent mouseEvent) {
    }

    /**
     * Mouse released event.
     * @param mouseEvent MouseEvent
     */
    public void mouseReleased(MouseEvent mouseEvent) {
        if (resizingDrag == true) {
            ModuleInfoNode mi = (ModuleInfoNode)this.getParent();
            mi.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        resizingDrag = false;
    }

    /**
     * Mouse clicked event. Synchronizes with the timeline if needed.
     * @param mouseEvent MouseEvent
     */
    public void mouseClicked(MouseEvent mouseEvent) {
        Rectangle r1 = m_activationRect.get("Icon.Enabled");

        if (r1 != null) {
            Point p = mouseEvent.getPoint();

            if (r1.contains(p)) {
                m_actuatorCurve.setActive(!m_actuatorCurve.isActive());
                synchronizeOpposite();
                this.repaint();
                getOpposite().repaint();
                return;
            }
        }

        Rectangle r2 = m_activationRect.get("Icon.Expanded");

        if (r2 != null) {
            Point p = mouseEvent.getPoint();

            if (r2.contains(p)) {
                setExpanded(!isExpanded());
                updateExpandedState();
                synchronizeOpposite();
                ModuleInfoNode mt = (ModuleInfoNode)this.getParent();
                mt.refreshComponentsLocation();

                ModuleTimelineNode oppParent = (ModuleTimelineNode) getOpposite().
                                               getParent();

                oppParent.refreshComponentsLocation();

                return;
            }
        }
    }

    /**
     * Mouse dragged event. Used when the window is expanded/resized.
     * @param mouseEvent MouseEvent
     */
    public void mouseDragged(MouseEvent mouseEvent) {
        if (!isExpanded()) {
            return;
        }

        int y = mouseEvent.getY();

        if (Math.abs(y - getExpandedHeight()) < 10) {
            resizingDrag = true;
            m_customHeightEnabled = true;
        }

        if (resizingDrag == true && y > getClosedHeight() * 3) {
            m_customHeight = y;

            setSize(getWidth(), y);
            updateExpandedState();
            synchronizeOpposite();

            ModuleInfoNode mt = (ModuleInfoNode)this.getParent();
            mt.setCursor(new Cursor(Cursor.S_RESIZE_CURSOR));
            mt.refreshComponentsLocation();
            ModuleTimelineNode oparent = (ModuleTimelineNode)
                                         getOpposite().
                                         getParent();
            oparent.refreshComponentsLocation();
            mt.repaint();
        }
    }

    /**
     * Mouse moved event. Used when the window is resized.
     * @param mouseEvent MouseEvent
     */
    public void mouseMoved(MouseEvent mouseEvent) {

        int y = mouseEvent.getY();

        if (Math.abs(y - getExpandedHeight()) < 6) {
            ModuleInfoNode mi = (ModuleInfoNode)this.getParent();
            mi.setCursor(new Cursor(Cursor.S_RESIZE_CURSOR));

        } else {
            ModuleInfoNode mi = (ModuleInfoNode)this.getParent();
            mi.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }

    }

    /**
     * Mouse entered event.
     * @param mouseEvent MouseEvent
     */
    public void mouseEntered(MouseEvent mouseEvent) {
        ModuleInfoNode mi = (ModuleInfoNode)this.getParent();
        mi.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    /**
     * Mouse exit event.
     * @param mouseEvent MouseEvent
     */
    public void mouseExited(MouseEvent mouseEvent) {
        ModuleInfoNode mi = (ModuleInfoNode)this.getParent();
        mi.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
}
