/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * TimelinesTab.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: ModuleTimelineNode.java,v 1.8 2005/01/26 22:41:43 drapel Exp $
 */


package birg.bluemove.timelines.gui.timelinestree;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.*;

import javax.swing.ImageIcon;

import birg.bluemove.module.core.*;
import birg.bluemove.resources.resources;
import birg.bluemove.timelines.gui.JTimelineComponent;
import birg.bluemove.timelines.gui.infotree.ModulesTree;
import birg.bluemove.timelines.gui.timelinestree.nodes.*;
import birg.bluemove.utils.gui.GUITools;
import org.apache.log4j.Logger;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: ModuleTimelineNode.java,v 1.8 2005/01/26 22:41:43 drapel Exp $
 */
public class ModuleTimelineNode extends JTimelineComponent implements
        MouseListener, Observer {

    static Logger logger = Logger.getLogger(ModuleTimelineNode.class);
    Module m_module = null;

    Vector<ModuleComponentTimelineNode> m_componentTimelineNodes = null;

    HashMap<String, Rectangle> m_activationRect;

    Color m_colorIsActive = new Color(0.75f, 1.0f, 0.7f);
    Color m_colorIsInactive = new Color(1.0f, 0.75f, 0.4f);

    /**
     * Constructor.
     * @param module Module
     */
    public ModuleTimelineNode(Module module) {
        this.setLayout(null);

        m_module = module;
        m_module.addObserver(this);
        m_componentTimelineNodes = new Vector<ModuleComponentTimelineNode>();

        // hashmap for activation regions (icons)
        m_activationRect = new HashMap<String, Rectangle>();
        this.addMouseListener(this);
        this.setToolTipText(m_module.getComment());
    }

    /**
     * Gets the module linked to this timeline group.
     * @return Module
     */
    public Module getModule() {
        return m_module;
    }

    /**
     * Adds a component timeline to this module.
     * @param component ModuleComponent
     */
    public void addComponent(ModuleComponent component) {
        ModuleComponentTimelineNode m = null;

        if (component instanceof Actuator) {
            m = new ActuatorTimelineNode(component);
        }

        if (component instanceof Sensor) {
            m = new SensorTimelineNode(component);
        }

        assert (m != null);

        m_componentTimelineNodes.add(m);
        this.add(m);
    }

    /**
     * Removes a component timeline from this module.
     * @param component ModuleComponent
     */
    public void removeComponent(ModuleComponent component) {
        for (ModuleComponentTimelineNode element : m_componentTimelineNodes) {
            if (element.getComponent() == component) {
                m_componentTimelineNodes.remove(element);
                this.remove(element);
                break;
            }
        }
    }


    /**
     * Paints border.
     * @param g Graphics
     */
    public void paintBorder(Graphics g) {
        Graphics2D g2d = (Graphics2D) g.create();
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        g2d.setColor(Color.BLACK);
        g2d.drawRect(bounds.x, bounds.y, bounds.width - 1, bounds.height);
        g2d.dispose();
    }


    /**
     * Paints a component
     * @param g Graphics
     */

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g.create();
        g2d.setRenderingHints(GUITools.getBestQualityHints());

        Rectangle bounds = new Rectangle(0, 0, getWidth(), getClosedHeight());

        GradientPaint gradient;

        if (m_module.getTimelineModule().isActive()) {
            gradient = new GradientPaint(0, 0, m_colorIsActive,
                                         0, this.getClosedHeight(), Color.WHITE,
                                         false);
        } else {
            gradient = new GradientPaint(0, 0, m_colorIsInactive,
                                         0, this.getClosedHeight(), Color.WHITE,
                                         false);
        }

        g2d.setPaint(gradient);
        g2d.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

        if (!m_module.getTimelineModule().isActive()) {
            g2d.setColor(Color.WHITE);
            for (int n = -bounds.height; n < bounds.width; n += 10) {
                g2d.drawLine(n, 0, n + bounds.height, bounds.height);
                g2d.drawLine(n + bounds.height, 0, n, bounds.height);
            }
        }
        g2d.dispose();
    }

    /**
     * Gets LED icon.
     * @return ImageIcon
     */
    protected ImageIcon getIconLed() {
        if (m_module.getTimelineModule().isActive()) {
            return new ImageIcon(resources.class.getResource(
                    "icon_greenlight.png"));
        } else {
            return new ImageIcon(resources.class.getResource(
                    "icon_redlight.png"));
        }
    }

    /**
     * Gets arrow icon.
     * @return ImageIcon
     */
    protected ImageIcon getIconArrow() {
        if (isExpanded()) {
            return new ImageIcon(resources.class.getResource(
                    "icon_arrowdown.png"));
        } else {
            return new ImageIcon(resources.class.getResource(
                    "icon_arrowright.png"));
        }
    }

    /**
     * Gets all components within this module.
     * @return Vector
     */
    public Vector<ModuleComponentTimelineNode> getComponentTimelineNodes() {
        return m_componentTimelineNodes;
    }

    /**
     * Mouse clicked event.
     * @param mouseEvent MouseEvent
     */
    public void mouseClicked(MouseEvent mouseEvent) {
        Rectangle r1 = m_activationRect.get("Icon.Enabled");

        if (r1 != null) {
            Point p = mouseEvent.getPoint();

            if (r1.contains(p)) {
                m_module.getTimelineModule().setActive(!m_module.
                        getTimelineModule().isActive());

                this.repaint();
                return;
            }
        }

        Rectangle r2 = m_activationRect.get("Icon.Expanded");

        if (r2 != null) {
            Point p = mouseEvent.getPoint();

            if (r2.contains(p)) {
                setExpanded(!isExpanded());
                updateExpandedState();
                synchronizeOpposite();
                ModulesTree mt = (ModulesTree)this.getParent();
                mt.refreshModulesLocation();
                mt.repaint();
                return;
            }
        }
    }

    /**
     * Gets expanded height.
     * @return int
     */
    public int getExpandedHeight() {
        int h = this.getClosedHeight();
        for (Component c : this.getComponents()) {
            h += c.getHeight();
        }
        return h;
    }

    /**
     * Refreshes components location.
     */
    public void refreshComponentsLocation() {
        int y = getClosedHeight();

        for (Component c : this.getComponents()) {
            c.setLocation(0, y);
            y += c.getHeight();
        }

        updateExpandedState();

        TimelinesTree mt = (TimelinesTree)this.getParent();
        mt.refreshModulesLocation();
        mt.invalidate();
        mt.repaint();
    }

    /**
     * Resets size and location of components.
     */
    public void resetSizeLocation() {

        int y = getClosedHeight();

        for (Component c : this.getComponents()) {
            ModuleComponentTimelineNode mctn = (ModuleComponentTimelineNode) c;
            c.setLocation(0, y);
            if (mctn.isExpanded()) {
                int height = mctn.getHeight();
                mctn.setCurrentDimension(getWidth(), height);
                y += height;
            } else {
                int height = getClosedHeight();
                mctn.setCurrentDimension(getWidth(), height);
                y += height;
            }
        }
    }


    public void mousePressed(MouseEvent mouseEvent) {
    }

    public void mouseReleased(MouseEvent mouseEvent) {
    }

    public void mouseEntered(MouseEvent mouseEvent) {
    }

    public void mouseExited(MouseEvent mouseEvent) {
    }

    /**
     * Updates when the module manager has changed.
     * @param o Observable
     * @param arg Object
     */
    public void update(Observable o, Object arg) {
        logger.debug("Update observer --> " + this.getClass().getName());
        if (o instanceof Module) {
            if (arg.toString().equals("add")) {
                ModuleComponent c = m_module.getLastComponent();
                this.addComponent(c);
            } else if (arg.toString().equals("remove")) {
                ModuleComponent c = m_module.getLastRemovedComponent();
                this.removeComponent(c);
            }
        }
    }


}
