/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * TimelinesTab.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: TimelinesTree.java,v 1.11 2005/01/26 22:41:43 drapel Exp $
 */


package birg.bluemove.timelines.gui.timelinestree;


import java.awt.Component;
import java.awt.Font;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import birg.bluemove.module.core.Module;
import birg.bluemove.module.core.ModulesManager;
import birg.bluemove.timelines.gui.TimelinesTab;
import birg.bluemove.timelines.gui.infotree.ModulesTree;
import birg.bluemove.timelines.gui.timelinestree.nodes.
        ModuleComponentTimelineNode;
import birg.bluemove.utils.gui.FontTools;


/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version $Id: TimelinesTree.java,v 1.11 2005/01/26 22:41:43 drapel Exp $
 */
public class TimelinesTree extends JComponent implements AncestorListener {

    ModulesManager m_modulesManager = null;
    Vector<ModuleTimelineNode> m_modulesTimelineNodes = new Vector<
            ModuleTimelineNode>();

    protected int m_verticalOffset = 0;
    protected TimelinesTab m_timelinesTab = null;

    protected ModulesTree m_modulesTree = null;

    /**
     *
     * @param modulesManager ModulesManager
     * @param tab TimelinesTab
     */
    public TimelinesTree(ModulesManager modulesManager, TimelinesTab tab) {
        m_modulesManager = modulesManager;
        m_timelinesTab = tab;
        this.addAncestorListener(this);
    }

    /**
     * Sets vertical offset for scrolling.
     * @param offset int
     */
    public void setVerticalOffset(int offset) {
        m_verticalOffset = offset;
    }


    /**
     * Retrieves vertical offset for scrolling.
     * @return int
     */
    public int getVerticalOffset() {
        return m_verticalOffset;
    }

    /**
     * Resets size and location of modules.
     */
    public void resetSizeLocation() {
        Font font = FontTools.getControlFont();
        int height = FontTools.getMaxHeight(font);
        int y = m_verticalOffset;

        for (Component c : this.getComponents()) {
            ModuleTimelineNode t = (ModuleTimelineNode) c;
            c.setLocation(0, y);
            if (t.isExpanded()) {
                c.setSize(getWidth(), t.getHeight());
                y += t.getHeight();
            } else {
                c.setSize(getWidth(), height * 3);
                y += height * 3;
            }
            t.resetSizeLocation();
        }
        m_timelinesTab.setScrollbarRange();
    }

    /**
     * Refreshes modules location.
     */
    public void refreshModulesLocation() {
        int y = m_verticalOffset;

        for (Component c : this.getComponents()) {
            c.setLocation(0, y);
            y += c.getHeight();
        }
        m_timelinesTab.setScrollbarRange();
    }

    /**
     * Add a new module.
     * @param module Module
     */
    public void addModule(Module module) {
        ModuleTimelineNode m = new ModuleTimelineNode(module);
        m_modulesTimelineNodes.add(m);
        this.add(m);
    }

    /**
     * Remove a module.
     * @param module Module
     */
    public void removeModule(Module module) {
        for (ModuleTimelineNode element : m_modulesTimelineNodes) {
            if (element.getModule() == module) {
                m_modulesTimelineNodes.remove(element);
                this.remove(element);
                break;
            }
        }
    }

    /**
     * Returns the current height.
     * @return int
     */
    public int getCurrentHeight() {
        int h = 0;
        for (Component c : this.getComponents()) {
            h += c.getHeight();
        }
        return h;
    }

    /**
     * Remove all modules.
     */
    public void clear() {
        for (ModuleTimelineNode element : m_modulesTimelineNodes) {
            this.remove(element);
        }
        m_modulesTimelineNodes.clear();
    }

    /**
     * Gets all modules in a vector.
     * @return Vector
     */
    public Vector<ModuleTimelineNode> getNodes() {
        return m_modulesTimelineNodes;
    }

    /**
     * Sets modulesTree (opposite tree).
     * @param tree ModulesTree
     */
    public void setModulesTree(ModulesTree tree) {
        m_modulesTree = tree;
    }

    /**
     * Repaints all components.
     */
    public void repaintComponents() {
        for (ModuleTimelineNode element : m_modulesTimelineNodes) {
            for (ModuleComponentTimelineNode comp :
                 element.getComponentTimelineNodes()) {
                comp.repaint();
            }
        }
    }

    /**
     * Ancestor added event. It is called when the tree is created
     * for the first time.
     * @param ancestorEvent AncestorEvent
     */
    public void ancestorAdded(AncestorEvent ancestorEvent) {
        resetSizeLocation();
        for (Component c : this.getComponents()) {
            ((ModuleTimelineNode) c).resetSizeLocation();
        }

    }

    public void ancestorRemoved(AncestorEvent ancestorEvent) {
    }

    public void ancestorMoved(AncestorEvent ancestorEvent) {
    }
}
