package birg.bluemove.timelines.gui.timelinestree.nodes;

import java.awt.*;
import java.awt.event.*;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.*;

import birg.bluemove.module.core.ModuleComponent;
import birg.bluemove.timelines.core.curve.ActuatorCurve;
import birg.bluemove.timelines.core.interpolator.Key;
import birg.bluemove.timelines.gui.TimelinesTab;
import birg.bluemove.utils.V2D;
import birg.bluemove.utils.gui.*;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class ActuatorTimelineNode extends ModuleComponentTimelineNode implements
        ActionListener {

    protected ActuatorCurve m_curve = null;
    protected ImageIcon m_iconHermite = null;
    protected ImageIcon m_iconLinear = null;

    protected Point m_currentMousePosition = null;

    protected Color m_colorIsActive = new Color(0.90f, 1.0f, 0.9f);
    protected Color m_colorIsInactive = new Color(1.0f, 0.90f, 0.6f);
    public static final int m_scaleMargin = 45;

    protected Key m_popupSelectedKey;

    protected HashMap<String, Rectangle> m_activationRect;

    protected JPopupMenu m_popup = null;

    protected boolean isDragging = false;
    protected Key draggedKey = null;

    /**
     * Constructor.
     * @param moduleComponent ModuleComponent
     */
    public ActuatorTimelineNode(ModuleComponent moduleComponent) {
        super(moduleComponent);

        m_iconHermite = GUITools.getImageIcon("timelinekey_spline.png");
        m_iconLinear = GUITools.getImageIcon("timelinekey_linear.png");

        m_curve = (ActuatorCurve) m_moduleComponent.getCurve();
        m_activationRect = new HashMap<String, Rectangle>();

        m_popup = new JPopupMenu();
    }

    /**
     * Paints the border.
     * @param g Graphics
     */
    public void paintBorder(Graphics g) {
        //super.paintBorder(g);
        Graphics2D g2d = (Graphics2D) g.create();
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        g2d.setColor(Color.BLACK);
        g2d.drawRect(bounds.x, bounds.y, bounds.width - 1, bounds.height);
        g2d.dispose();
    }

    /**
     * Paints the component.
     * @param g Graphics
     */
    public void paintComponent(Graphics g) {
        //super.paintComponent(g);

        Graphics2D g2d = (Graphics2D) g.create();
        g2d.setRenderingHints(GUITools.getBestQualityHints());

        GradientPaint gradient;

        gradient = new GradientPaint(0, 0, m_colorIsInactive,
                                     getWidth(), 0,
                                     Color.WHITE,
                                     false);

        g2d.setPaint(gradient);
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        int hg = getClosedHeight();
        g2d.fillRect(bounds.x, bounds.y, bounds.width, hg / 2);

        if (!isExpanded()) {
            return;
        }

        g2d.setColor(Color.BLACK);
        g2d.drawLine(bounds.x, hg, bounds.x + bounds.width, hg);

        Rectangle lowerRect = new Rectangle(bounds.x, hg + 1, bounds.width,
                                            bounds.height - hg - 1);
        g2d.setColor(Color.WHITE);
        g2d.fillRect(lowerRect.x, lowerRect.y, lowerRect.width,
                     lowerRect.height);

        int realHeight = lowerRect.height - m_scaleMargin;

        Rectangle scaleRect = new Rectangle(lowerRect.x,
                                            lowerRect.y +
                                            (lowerRect.height -
                                             realHeight) /
                                            2,
                                            lowerRect.width, realHeight);

        // if (realHeight > 25) {

        g2d.setColor(Color.BLACK);

        g2d.drawLine(m_scaleMargin, scaleRect.y, m_scaleMargin,
                     scaleRect.y + scaleRect.height);

        drawScaleLine(g2d, scaleRect.x, scaleRect.x + scaleRect.width,
                      scaleRect.y,
                      new Double("1.0"));
        drawScaleLine(g2d, scaleRect.x, scaleRect.x + scaleRect.width,
                      scaleRect.y + scaleRect.height,
                      new Double("-1.0"));
        // }

        drawScale(g2d, scaleRect, scaleRect.height, new Double("-1.0"),
                  new Double("1.0"));

        m_activationRect.put("Rect.Scale", scaleRect);

        Rectangle curveRect = new Rectangle(scaleRect);
        curveRect.x += m_scaleMargin;
        curveRect.width -= m_scaleMargin;

        m_activationRect.put("Rect.Curve", curveRect);

        drawCurve(g2d);

        // draw vertical marker
        if (m_currentMousePosition != null) {
            if (curveRect.contains(m_currentMousePosition)) {
                g2d.setColor(Color.BLACK);
                g2d.drawLine(m_currentMousePosition.x, curveRect.y,
                             m_currentMousePosition.x,
                             curveRect.y + curveRect.height);

            }
        }

        g2d.dispose();
    }

    /**
     * Draws the curve.
     * @param g2d Graphics2D
     */
    private void drawCurve(Graphics2D g2d) {

        Vector<Key> keys = m_curve.getKeys();

        if (m_curve.getNbKeys() >= 2) {
            m_curve.computeTangents(1.0);

            Rectangle drawingRect = m_activationRect.get("Rect.Curve");
            int yEnd = drawingRect.y + drawingRect.height;

            Vector<Integer> xPoints = new Vector<Integer>();
            Vector<Integer> yPoints = new Vector<Integer>();

            for (double px = drawingRect.x;
                             px < drawingRect.x + drawingRect.width; px += 2.0) {

                // mapping from pixels space to frame space
                double frame = m_timeScale.getFrameFromPixel((int) px);
                xPoints.add((int) px);
                double value = m_curve.evaluate(frame);
                value = Math.max(0.0, value);
                value = Math.min(1.0, value);
                yPoints.add((int) (yEnd - value * (double) drawingRect.height));
            }

            g2d.setColor(Color.BLACK);

            int[] xPointsArray = new int[xPoints.size()];
            int[] yPointsArray = new int[yPoints.size()];

            for (int n = 0; n < xPointsArray.length; n++) {
                xPointsArray[n] = xPoints.get(n);
                yPointsArray[n] = yPoints.get(n);
            }

            Stroke oldStroke = g2d.getStroke();
            g2d.setStroke(new BasicStroke(1.5f));
            g2d.drawPolyline(xPointsArray, yPointsArray, xPointsArray.length);
            g2d.setStroke(oldStroke);
        }

        Rectangle drawingRect = m_activationRect.get("Rect.Curve");
        int yEnd = drawingRect.y + drawingRect.height;

        for (Key k : keys) {
            V2D pos = k.getCoordinates();

            pos.x = m_timeScale.getPixelFromFrame(pos.x);
            pos.y = (double) yEnd - pos.y * (double) drawingRect.height;

            /* GRAPHICAL DEBUG OF TANGENTS
                         V2D tg1 = k.getTangentInput();
                         V2D tg2 = k.getTangentOutput();

                         tg1.x = pos.x - tg1.x;
                         tg1.y = pos.y - tg1.y;

                         tg2.x += pos.x;
                         tg2.y += pos.y;



                         tg1.x = m_timeScale.getPixelFromFrame(tg1.x);
             tg1.y = (double) yEnd - tg1.y * (double) drawingRect.height;

                         tg2.x = m_timeScale.getPixelFromFrame(tg2.x);
             tg2.y = (double) yEnd - tg2.y * (double) drawingRect.height;
             */

            if (pos.x < drawingRect.x ||
                pos.x > drawingRect.x + drawingRect.width) {
                continue;
            }

            /*
                        GUITools.drawIcon(g2d, m_iconLinear, (int) tg1.x - 6,
                                          (int) tg1.y - 6);

                        GUITools.drawIcon(g2d, m_iconLinear, (int) tg2.x - 6,
                                          (int) tg2.y - 6);

             g2d.drawLine((int) pos.x, (int) pos.y, (int) tg1.x, (int) tg1.y);
             g2d.drawLine((int) pos.x, (int) pos.y, (int) tg2.x, (int) tg2.y);
             */

            if (k.getKeyType() == Key.KeyType.HERMITE) {
                GUITools.drawIcon(g2d, m_iconHermite, (int) pos.x - 6,
                                  (int) pos.y - 6);
            } else {
                GUITools.drawIcon(g2d, m_iconLinear, (int) pos.x - 6,
                                  (int) pos.y - 6);
            }
        }

    }

    /**
     * Draws scale.
     * @param g2d Graphics2D
     * @param curveZone Rectangle
     * @param originalHeight int
     * @param startValue Double
     * @param endValue Double
     */
    private void drawScale(Graphics2D g2d, Rectangle curveZone,
                           int originalHeight, Double startValue,
                           Double endValue) {

        int height = curveZone.height;

        if (height <= 35) {
            return;
        }

        float ratio = 1.0f - ((float) height / (float) originalHeight);
        g2d.setColor(new Color(ratio, ratio, ratio));

        int yPos = curveZone.y + height / 2;
        double middle = (startValue + endValue) * 0.5;

        drawScaleLine(g2d, curveZone.x, curveZone.x + curveZone.width, yPos,
                      new Double(middle));
        Rectangle lower = new Rectangle(curveZone.x, curveZone.y,
                                        curveZone.width, curveZone.height / 2);
        Rectangle upper = new Rectangle(curveZone.x, curveZone.y,
                                        curveZone.width, curveZone.height / 2);
        upper.translate(0, lower.height);

        drawScale(g2d, lower, originalHeight, startValue,
                  (startValue + endValue) * 0.5f);
        drawScale(g2d, upper, originalHeight, (startValue + endValue) * 0.5f,
                  endValue);
    }

    /**
     * Draws the scale grid.
     * @param g2d Graphics2D
     * @param x1 int
     * @param x2 int
     * @param y int
     * @param value Double
     */
    protected void drawScaleLine(Graphics2D g2d, int x1, int x2, int y,
                                 Double value) {

        int fHeight = FontTools.getMaxHeight(FontTools.getControlFont());

        g2d.drawLine(x1 + m_scaleMargin - 6, y,
                     x2,
                     y);

        DecimalFormat df = new DecimalFormat("##0.00");

        g2d.setColor(Color.BLACK);
        g2d.drawString("" + df.format(value), 5, y + fHeight / 2);
    }

    /**
     * Mouse moved event.
     * @param mouseEvent MouseEvent
     */
    public void mouseMoved(MouseEvent mouseEvent) {
        m_currentMousePosition = mouseEvent.getPoint();
        m_timeScale.setIndicatorPos(mouseEvent.getPoint());
        this.repaint();
    }

    /**
     * Mouse clicked event.
     * @param mouseEvent MouseEvent
     */
    public void mouseClicked(MouseEvent mouseEvent) {
    }

    /**
     * Normalizes the height value between 0 and 1.0.
     * @param value double
     * @param rect Rectangle
     * @return double
     */
    public double normalize(double value, Rectangle rect) {
        double h = (double) rect.height;

        double nh = 1.0 - ((value - (double) rect.y) / h);
        return nh;
    }

    /**
     * Unormalizes the height value according to the rectangle.
     * @param value double
     * @param rect Rectangle
     * @return double
     */
    public double unormalize(double value, Rectangle rect) {
        double h = (double) rect.height;

        double nh = value * h;
        return (double) (rect.y - nh + rect.height);
    }


    /**
     * Mouse dragged event.
     * @param mouseEvent MouseEvent
     */
    public void mouseDragged(MouseEvent mouseEvent) {
        if (isDragging == true) {
            m_currentMousePosition = mouseEvent.getPoint();

            int x = m_currentMousePosition.x;
            int y = m_currentMousePosition.y;

            Rectangle rc = m_activationRect.get("Rect.Curve");

            // the point can't be dragged outside from the drawing zone
            x = Math.max(x, rc.x);
            x = Math.min(x, rc.x + rc.width);

            y = Math.max(y, rc.y);
            y = Math.min(y, rc.y + rc.height);

            double value = normalize(y, rc);
            double frame = m_timeScale.getFrameFromPixel(x);

            draggedKey.setCoordinates(new V2D(frame, value));
            m_timeScale.setIndicatorPos(mouseEvent.getPoint());

            this.repaint();
            return;

        }

        Vector<Key> keys = m_curve.getKeys();

        for (Key k : keys) {
            V2D kCoord = k.getCoordinates();
            double y = unormalize(kCoord.y, m_activationRect.get("Rect.Curve"));
            double x = m_timeScale.getPixelFromFrame(kCoord.x);

            Point pos = mouseEvent.getPoint();
            if (Point.distance(x, y, pos.x, pos.y) < 10) {
                if (!isDragging) {
                    draggedKey = k;
                    isDragging = true;
                    return;
                }
            }
        }

    }

    /**
     * Returns the key which is, to some extent, under the mouse.
     * @param mousePos Point
     * @return Key Returns the key under/near the mouse or null if no key
     */
    protected Key getKeyUnderMouse(Point mousePos) {
        Vector<Key> keys = m_curve.getKeys();

        for (Key k : keys) {
            V2D kCoord = k.getCoordinates();
            double y = unormalize(kCoord.y,
                                  m_activationRect.get("Rect.Curve"));
            double x = m_timeScale.getPixelFromFrame(kCoord.x);

            if (Point.distance(x, y, mousePos.x, mousePos.y) < 10) {
                return k;
            }
        }

        return null;
    }

    /**
     * Mouse pressed event.
     * @param mouseEvent MouseEvent
     */
    public void mousePressed(MouseEvent mouseEvent) {
        Rectangle drawingZone = m_activationRect.get("Rect.Curve");

        if (drawingZone != null) {
            Point p = mouseEvent.getPoint();

            Key keyUnderMouse = getKeyUnderMouse(p);

            /**
             * ADD KEY
             */
            if (drawingZone.contains(p) &&
                mouseEvent.getButton() == mouseEvent.BUTTON1 && keyUnderMouse == null) {
                double value = normalize(p.y, drawingZone);
                double frame = m_timeScale.getFrameFromPixel(mouseEvent.
                        getX());
                if (TimelinesTab.m_currentToolType.equals(TimelinesTab.ToolType.
                        ADD_SPLINE)) {
                    m_curve.addKey(new Key(Key.KeyType.HERMITE, frame, value));
                } else {
                    m_curve.addKey(new Key(Key.KeyType.LINEAR, frame, value));
                }

                this.repaint();
                return;
            }

            if (drawingZone.contains(p) &&
                mouseEvent.getButton() == mouseEvent.BUTTON3) {
                if (keyUnderMouse != null) {
                    m_popupSelectedKey = keyUnderMouse;
                    m_popup.removeAll();

                    JMenuItem jm = new JMenuItem("delete key");
                    jm.addActionListener(this);
                    m_popup.add(jm);

                    if (keyUnderMouse.getKeyType() == Key.KeyType.LINEAR) {
                        jm = new JMenuItem("convert to spline key");
                    } else {
                        jm = new JMenuItem("convert to linear key");
                    }

                    jm.addActionListener(this);
                    m_popup.add(jm);

                    m_popup.show(this, p.x + 7, p.y);
                    return;
                } else {

                    m_popupSelectedKey = keyUnderMouse;
                    m_popup.removeAll();

                    JMenuItem jm = new JMenuItem("remove all keys");
                    jm.addActionListener(this);
                    m_popup.add(jm);

                    jm = new JMenuItem("convert all keys to linear keys");
                    jm.addActionListener(this);
                    m_popup.add(jm);

                    jm = new JMenuItem("convert all keys to spline keys");
                    jm.addActionListener(this);
                    m_popup.add(jm);

                    m_popup.show(this, p.x + 7, p.y);
                }
            }
        }
    }

    /**
     * Mouse released event.
     * @param mouseEvent MouseEvent
     */
    public void mouseReleased(MouseEvent mouseEvent) {
        isDragging = false;
    }

    /**
     * Mouse entered event.
     * @param mouseEvent MouseEvent
     */
    public void mouseEntered(MouseEvent mouseEvent) {
    }

    /**
     * Mouse exit ev.
     * @param mouseEvent MouseEvent
     */
    public void mouseExited(MouseEvent mouseEvent) {
    }

    /**
     * Action performed listener. Catches the events fired by the
     * popup menu.
     * @param e ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() instanceof JMenuItem) {

            JMenuItem item = (JMenuItem) e.getSource();
            if (item.getText().equals("delete key")) {
                m_curve.removeKey(m_popupSelectedKey);
                this.repaint();
                return;
            }

            if (item.getText().equals("convert to linear key")) {
                m_popupSelectedKey.setKeyType(Key.KeyType.LINEAR);
                this.repaint();
                return;
            }

            if (item.getText().equals("convert to spline key")) {
                m_popupSelectedKey.setKeyType(Key.KeyType.HERMITE);
                this.repaint();
                return;
            }

            if (item.getText().equals("remove all keys")) {
                MessageBox mb = new MessageBox(null,
                                               MessageBox.MessageBoxType.
                                               QUESTION,
                                               "All keys will be removed from this timeline, do you want to continue ?");
                mb.showDialog();
                if (mb.getAnswer() == MessageBox.MessageBoxAnswer.YES) {
                    m_curve.getKeys().clear();
                    this.repaint();
                }
                return;
            }

            if (item.getText().equals("convert all keys to spline keys")) {
                for (Key k : m_curve.getKeys()) {
                    k.setKeyType(Key.KeyType.HERMITE);
                }
                this.repaint();
                return;
            }

            if (item.getText().equals("convert all keys to linear keys")) {
                for (Key k : m_curve.getKeys()) {
                    k.setKeyType(Key.KeyType.LINEAR);
                }
                this.repaint();
                return;
            }

        }

    }


}
