package birg.bluemove.timelines.gui.timescale;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;

import javax.swing.JComponent;

import birg.bluemove.timelines.gui.infotree.ModulesTree;
import birg.bluemove.timelines.gui.timelinestree.TimelinesTree;
import birg.bluemove.timelines.gui.timelinestree.nodes.ActuatorTimelineNode;
import birg.bluemove.utils.gui.GUITools;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class TimeScale extends JComponent implements MouseMotionListener,
        MouseListener {

    Color m_gradientColor = new Color(0.8f, 0.8f, 0.8f);

    Point m_indicatorPos = null;

    public static final int zoomLevel[] = {3, 10, 20, 40, 80, 160};

    double m_startFrame = 0.0;

    // for 1 frame -> 25 pixels (1.0/25.0)
    double m_frameToPixelRatio = 0.0;

    protected int currentZoom = 2;

    protected ModulesTree m_modulesTree = null;
    protected TimelinesTree m_timelinesTree = null;

    protected Point startDraggingPoint = null;
    protected int startDraggingFrame = 0;
    protected boolean isDragging = false;


    /**
     * Constructor.
     * @param mt ModulesTree
     * @param tt TimelinesTree
     */
    public TimeScale(ModulesTree mt, TimelinesTree tt) {
        this.addMouseListener(this);
        this.addMouseMotionListener(this);

        m_modulesTree = mt;
        m_timelinesTree = tt;

        setPixelsPerFrame(zoomLevel[currentZoom]);
    }

    /**
     * Increases the zoom factor.
     */
    public void increaseZoom() {
        if (currentZoom > 0) {
            currentZoom--;
            setPixelsPerFrame(zoomLevel[currentZoom]);
        }
    }

    /**
     * Decreases the zoom factor.
     */
    public void decreaseZoom() {
        if (currentZoom < zoomLevel.length - 1) {
            currentZoom++;
            setPixelsPerFrame(zoomLevel[currentZoom]);
        }
    }

    /**
     * Gets start frame.
     * @return double
     */
    public double getStartFrame() {
        return m_startFrame;
    }


    /**
     * Sets start frame.
     * @return double
     */
    public void setStartFrame(double frame) {
        m_startFrame = frame;
    }

    /**
     * Gets ratio between pixels and frames.
     * @return double
     */
    public double getPixelsPerFrame() {
        return 1.0 / m_frameToPixelRatio;
    }

    /**
     * Sets ratio between pixels and frames.
     * @param pixels int
     */
    public void setPixelsPerFrame(int pixels) {
        m_frameToPixelRatio = 1.0 / (double) pixels;
    }

    /**
     * Gets a frame value from a pixel position.
     * @param px int
     * @return double
     */
    public double getFrameFromPixel(int px) {
        int x = px - ActuatorTimelineNode.m_scaleMargin;
        return (double) m_startFrame + m_frameToPixelRatio * (double) x;
    }

    /**
     * Gets a pixel position from a frame value.
     * @param frame double
     * @return double
     */
    public double getPixelFromFrame(double frame) {
        double f = frame - (double) m_startFrame;

        return (double) ActuatorTimelineNode.m_scaleMargin +
                (double) f / m_frameToPixelRatio;
    }


    /**
     * Paints border.
     * @param g Graphics
     */
    public void paintBorder(Graphics g) {
        super.paintBorder(g);
        Graphics2D g2d = (Graphics2D) g.create();
        Rectangle bounds = g2d.getClipBounds();
        g2d.setColor(Color.BLACK);
        g2d.drawRect(bounds.x, bounds.y, bounds.width - 1, bounds.height - 1);
        g2d.dispose();
    }

    /**
     * Paints component.
     * @param g Graphics
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2d = (Graphics2D) g.create();
        g2d.setRenderingHints(GUITools.getBestQualityHints());

        GradientPaint gradient;

        gradient = new GradientPaint(0, 0, m_gradientColor,
                                     0, getHeight(),
                                     Color.WHITE,
                                     false);

        g2d.setPaint(gradient);
        Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
        g2d.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

        g2d.setColor(Color.BLACK);

        int x;
        double currentFrame;
        int tick;

        if (zoomLevel[currentZoom] > 5) {
            g2d.setColor(new Color(140, 140, 140));
            x = ActuatorTimelineNode.m_scaleMargin;
            currentFrame = m_startFrame;
            tick = (int) currentFrame;

            while (x < getWidth()) {

                tick = getNextTick((int) currentFrame, 1);

                // convert frames to pixel
                double f = (double) currentFrame - m_startFrame;

                x = (int) (ActuatorTimelineNode.m_scaleMargin +
                           f / m_frameToPixelRatio);

                g2d.drawLine(x, 0, x, 3);
                currentFrame = tick;

            }
        }

        if (zoomLevel[currentZoom] >= 5) {
            g2d.setColor(new Color(80, 80, 80));
            x = ActuatorTimelineNode.m_scaleMargin;
            currentFrame = m_startFrame;
            tick = (int) currentFrame;

            while (x < getWidth()) {
                if (currentFrame == 0) {
                    tick = 0;
                    currentFrame++;
                } else {
                    tick = getNextTick((int) currentFrame, 5);
                    currentFrame = tick;
                }

                // convert frames to pixel
                double f = (double) tick - m_startFrame;

                x = (int) (ActuatorTimelineNode.m_scaleMargin +
                           f / m_frameToPixelRatio);

                g2d.drawLine(x, 0, x, 10);

                String label = "" + tick;
                Rectangle2D r2d = g2d.getFontMetrics().getStringBounds(label,
                        g2d);

                g2d.drawString(label, x - (int) r2d.getWidth() / 2,
                               15 + (int) r2d.getHeight());

            }
        }

        g2d.setColor(Color.BLACK);
        x = ActuatorTimelineNode.m_scaleMargin;
        currentFrame = m_startFrame;
        tick = (int) currentFrame;

        x = ActuatorTimelineNode.m_scaleMargin;
        currentFrame = m_startFrame;
        tick = (int) currentFrame;

        while (x < getWidth()) {
            if (currentFrame == 0) {
                tick = 0;
                currentFrame++;
            } else {
                tick = getNextTick((int) currentFrame, 25);
                currentFrame = tick;
            }

            // convert frames to pixel
            double f = (double) tick - m_startFrame;

            x = (int) (ActuatorTimelineNode.m_scaleMargin +
                       f / m_frameToPixelRatio);

            g2d.drawLine(x, 0, x, 10);

            String label = "" + tick;
            Rectangle2D r2d = g2d.getFontMetrics().getStringBounds(label,
                    g2d);

            g2d.drawString(label, x - (int) r2d.getWidth() / 2,
                           15 + (int) r2d.getHeight());

        }

        /**
         * Draw a small triangle showing the tick position.
         */

        if (m_indicatorPos != null &&
            m_indicatorPos.x >= ActuatorTimelineNode.m_scaleMargin &&
            !isDragging) {
            g2d.drawLine(m_indicatorPos.x, 1 + 4, m_indicatorPos.x + 5, 6 + 4);
            g2d.drawLine(m_indicatorPos.x, 1 + 4, m_indicatorPos.x - 5, 6 + 4);
            g2d.drawLine(m_indicatorPos.x - 5, 6 + 4, m_indicatorPos.x + 5,
                         6 + 4);
        }
        g2d.dispose();
    }

    /**
     * Takes the next integer for a given interval length.
     * @param currentFrame int
     * @param intervalLength int
     * @return int
     */
    public int getNextTick(int currentFrame, int intervalLength) {
        if (currentFrame % intervalLength == 0) {
            return currentFrame + intervalLength;
        } while (currentFrame % intervalLength != 0) {
            currentFrame++;
        }

        return currentFrame;
    }


    /**
     * Mouse Dragged event.
     * @param e MouseEvent
     */
    public void mouseDragged(MouseEvent e) {
        if (!isDragging) {
            return;
        }
        Point newPoint = e.getPoint();
        int xDiff = newPoint.x - startDraggingPoint.x;

        //  newPoint.x, on doit avoir la frame du point de dpart
        /*
                m_startFrame = (int) (startDraggingFrame -
                                      (float) newPoint.x /
                                      (m_scaleFactor / (float) m_rangeFrame));
         */
        m_startFrame = (startDraggingFrame -
                        (double) (newPoint.x -
                                  ActuatorTimelineNode.m_scaleMargin) *
                        m_frameToPixelRatio);
        if (m_startFrame < 0.0) {
            m_startFrame = 0.0;
        }

        m_timelinesTree.repaintComponents();

        this.repaint();
    }


    public void mouseMoved(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    /**
     * Mouse pressed event.
     * @param e MouseEvent
     */
    public void mousePressed(MouseEvent e) {
        if (!isDragging) {
            isDragging = true;
            startDraggingPoint = e.getPoint();
            startDraggingFrame = (int) getFrameFromPixel((int) e.getPoint().
                    getX());
        }
    }

    public void mouseReleased(MouseEvent e) {
        isDragging = false;
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    /**
     * Sets indicator position. This will later print a small triangle
     * to indicate where the cursor is located.
     * @param mousePosition Point
     */
    public void setIndicatorPos(Point mousePosition) {
        m_indicatorPos = mousePosition;
        this.repaint();
    }

}
