/*
 *     __   __
 *    / /  / /_ _____ __ _  ___ _  _____
 *   / _ \/ / // / -_)  ' \/ _ \ / / -_)
 *  /_.__/_/\_,_/\__/_/_/_/\___/___/\__/
 *
 * GUITools.java
 *
 * See "License.txt" for information on license agreement,
 * usage, redistribution and warranties.
 *
 * $Id: GUITools.java,v 1.6 2005/01/26 22:41:43 drapel Exp $
 */

package birg.bluemove.utils.gui;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import birg.bluemove.resources.resources;

/**
 * <p>Title: Bluemove</p>
 *
 * <p>Description: Modular Robot Project</p>
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: BIRG</p>
 *
 * @author Kevin Drapel
 * @version 1.0
 */
public class GUITools {

    /**
     * Constructor.
     */
    private GUITools() {
    }

    /**
     * Returns an ImageIcon from the /resources dir.
     * @param fName String
     * @return ImageIcon
     */
    public static ImageIcon getImageIcon(String fName) {
        return new ImageIcon(resources.class.getResource(fName));
    }

    /**
     * Returns an Image (from any path).
     * @param fName String Full Path
     * @return Image
     */
    public static Image getImage(String fName) {
        return getImageIcon(fName).getImage();
    }

    /**
     * Returns a BufferedImage (from any path).
     * @param fName String Full Path
     * @return BufferedImage
     */
    public static BufferedImage getBufferedImage(String fName) {
        return toBufferedImage(getImage(fName));
    }

    /**
     * Converts an Image to a BufferedImage.
     * @param image Image
     * @return BufferedImage
     */
    public static BufferedImage toBufferedImage(Image image) {
        // We test is the image is already a BufferedImage instance
        if (image instanceof BufferedImage) {
            // Nothing to do
            return ((BufferedImage) image);
        } else {
            // We assure that the image is completly loaded
            image = new ImageIcon(image).getImage();

            // We create the new image
            BufferedImage bufferedImage = new BufferedImage(
                    image.getWidth(null),
                    image.getHeight(null),
                    BufferedImage.TYPE_INT_RGB);
            Graphics g = bufferedImage.createGraphics();
            g.drawImage(image, 0, 0, null);
            g.dispose();

            return (bufferedImage);
        }
    }

    /**
     * Draws an icon in a given rectangle.
     * @param g2d Graphics2D
     * @param icon ImageIcon
     * @param r Rectangle
     */
    public static void drawIcon(Graphics2D g2d, ImageIcon icon, Rectangle r) {
        g2d.drawImage(icon.getImage(), r.x, r.y, r.width, r.height,
                      icon.getImageObserver());
    }

    /**
     * Draws an icon at a given position.
     * @param g2d Graphics2D
     * @param icon ImageIcon
     * @param x int
     * @param y int
     * @return Rectangle
     */
    public static Rectangle drawIcon(Graphics2D g2d, ImageIcon icon, int x,
                                     int y) {
        Rectangle r = new Rectangle(x, y, icon.getIconWidth(),
                                    icon.getIconHeight());
        drawIcon(g2d, icon, r);
        return r;
    }

    /**
     * Reads a BufferedImage from an InputStream.
     * @param iStream InputStream
     * @return BufferedImage
     * @throws IOException
     */
    public static BufferedImage readImage(InputStream iStream) throws
            IOException {
        return ImageIO.read(iStream);
    }

    /**
     * Resizes an image with area averaging (medium/high quality)
     * @param img BufferedImage
     * @param width int
     * @param height int
     * @return BufferedImage
     */
    public static BufferedImage resizeImage(BufferedImage img, int width,
                                            int height) {
        Image resized = ((Image) img).getScaledInstance(width, height,
                Image.SCALE_AREA_AVERAGING);
        BufferedImage bi = new BufferedImage(width, height,
                                             BufferedImage.TYPE_INT_RGB);
        Graphics2D big = bi.createGraphics();
        big.drawImage(resized, 0, 0, null);
        return bi;
    }

    /**
     * Sets highest quality for rendering.
     * @return RenderingHints
     */
    public static RenderingHints getBestQualityHints() {
        RenderingHints renderHints =
                new RenderingHints(RenderingHints.KEY_ANTIALIASING,
                                   RenderingHints.VALUE_ANTIALIAS_ON);
        renderHints.put(RenderingHints.KEY_RENDERING,
                        RenderingHints.VALUE_RENDER_QUALITY);

        return renderHints;
    }


}
